# Copyright (c) 2024 Vladimir Botka <vbotka@gmail.com>
# Copyright (c) 2024 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: remove_keys
short_description: Remove specific keys from dictionaries in a list
version_added: "9.1.0"
author:
  - Vladimir Botka (@vbotka)
  - Felix Fontein (@felixfontein)
description: This filter removes only specified keys from a provided list of dictionaries.
options:
  _input:
    description:
      - A list of dictionaries.
      - Top level keys must be strings.
    type: list
    elements: dictionary
    required: true
  target:
    description:
      - A single key or key pattern to remove, or a list of keys or keys patterns to remove.
      - If O(matching_parameter=regex) there must be exactly one pattern provided.
    type: raw
    required: true
  matching_parameter:
    description: Specify the matching option of target keys.
    type: str
    default: equal
    choices:
      equal: Matches keys of exactly one of the O(target) items.
      starts_with: Matches keys that start with one of the O(target) items.
      ends_with: Matches keys that end with one of the O(target) items.
      regex:
        - Matches keys that match the regular expresion provided in O(target).
        - In this case, O(target) must be a regex string or a list with single regex string.
"""

EXAMPLES = r"""
- l:
    - {k0_x0: A0, k1_x1: B0, k2_x2: [C0], k3_x3: foo}
    - {k0_x0: A1, k1_x1: B1, k2_x2: [C1], k3_x3: bar}

  # 1) By default match keys that equal any of the items in the target.
- t: [k0_x0, k1_x1]
  r: "{{ l | community.general.remove_keys(target=t) }}"

  # 2) Match keys that start with any of the items in the target.
- t: [k0, k1]
  r: "{{ l | community.general.remove_keys(target=t, matching_parameter='starts_with') }}"

  # 3) Match keys that end with any of the items in target.
- t: [x0, x1]
  r: "{{ l | community.general.remove_keys(target=t, matching_parameter='ends_with') }}"

  # 4) Match keys by the regex.
- t: ['^.*[01]_x.*$']
  r: "{{ l | community.general.remove_keys(target=t, matching_parameter='regex') }}"

  # 5) Match keys by the regex.
- t: '^.*[01]_x.*$'
  r: "{{ l | community.general.remove_keys(target=t, matching_parameter='regex') }}"

  # The results of above examples 1-5 are all the same.
- r:
    - {k2_x2: [C0], k3_x3: foo}
    - {k2_x2: [C1], k3_x3: bar}

  # 6) By default match keys that equal the target.
- t: k0_x0
  r: "{{ l | community.general.remove_keys(target=t) }}"

  # 7) Match keys that start with the target.
- t: k0
  r: "{{ l | community.general.remove_keys(target=t, matching_parameter='starts_with') }}"

  # 8) Match keys that end with the target.
- t: x0
  r: "{{ l | community.general.remove_keys(target=t, matching_parameter='ends_with') }}"

  # 9) Match keys by the regex.
- t: '^.*0_x.*$'
  r: "{{ l | community.general.remove_keys(target=t, matching_parameter='regex') }}"

  # The results of above examples 6-9 are all the same.
- r:
    - {k1_x1: B0, k2_x2: [C0], k3_x3: foo}
    - {k1_x1: B1, k2_x2: [C1], k3_x3: bar}
"""

RETURN = r"""
_value:
  description: The list of dictionaries with selected keys removed.
  type: list
  elements: dictionary
"""

from ansible_collections.community.general.plugins.plugin_utils._keys_filter import (
    _keys_filter_params,
    _keys_filter_target_str,
)


def remove_keys(data, target=None, matching_parameter="equal"):
    """remove specific keys from dictionaries in a list"""

    # test parameters
    _keys_filter_params(data, matching_parameter)
    # test and transform target
    tt = _keys_filter_target_str(target, matching_parameter)

    if matching_parameter == "equal":

        def keep_key(key):
            return key not in tt
    elif matching_parameter == "starts_with":

        def keep_key(key):
            return not key.startswith(tt)
    elif matching_parameter == "ends_with":

        def keep_key(key):
            return not key.endswith(tt)
    elif matching_parameter == "regex":

        def keep_key(key):
            return tt.match(key) is None

    return [{k: v for k, v in d.items() if keep_key(k)} for d in data]


class FilterModule:
    def filters(self):
        return {
            "remove_keys": remove_keys,
        }



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_containerregistrytoken_info
version_added: "3.7.0"
short_description: Get Azure Container Registry Token facts
description:
    - Get or list facts for Container Registry Token.

options:
    resource_group:
        description:
            - The name of the resource group to which the container registry belongs.
        type: str
        required: true
    registry_name:
        description:
            - The name of the container registry.
        type: str
        required: true
    name:
        description:
            - The name of the container registry token.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get instance of Registry Token
  azure_rm_containerregistrytoken_info:
    resource_group: myResourceGroup
    registry_name: myRegistry
    name: myToken

- name: List instances of Registry Token
  azure_rm_containerregistrytoken_info:
    resource_group: myResourceGroup
    registry_name: myRegistry
'''

RETURN = '''
registry_tokens:
    description:
        - A list of dictionaries containing facts for token.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.ContainerRegistry/registries/registry01/tokens/token01"
        name:
            description:
                - The name of the resource.
            returned: always
            type: str
            sample: token01
        registry_name:
            description:
                - The name of the container registry.
            type: str
            returned: always
            sample: registry01
        resource_group:
            description:
                - The name fo the resource group.
            type: str
            returned: always
            sample: myRG
        creation_date:
            description:
                - The creation date of scope map.
            type: str
            returned: always
            sample: '2025-07-21T03:16:03.828541Z'
        credentials:
            description:
                - The credentials that can be used for authenticating the token.
            type: str
            returned: always
            sample: {
                    "passwords": [
                        {
                            "creation_time": "2025-07-21T03:17:14.528612Z",
                            "expiry": "2026-07-22T03:17:00.551Z",
                            "name": "password1"
                        }
                    ]
                }
        provisioning_state:
            description:
                - Provisioning state of the container registry token.
            returned: always
            type: str
            sample: Succeeded
        scope_map_id:
            description:
                - The resource ID of the scope map to which the token will be associated with.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/myRG/providers/Microsoft.ContainerRegistry/registries/registry01/scopeMaps/map01"
        status:
            description:
                - The status of the token example enabled or disabled.
            type: str
            returned: always
            sample: enabled
        system_data:
            description:
                - Metadata pertaining to creation and last modification of the resource.
            type: dict
            returned: always
            sample: {
                    "created_at": "2025-07-21T03:16:50.586036Z",
                    "created_by": "test@domain.com",
                    "created_by_type": "User",
                    "last_modified_at": "2025-07-21T03:16:50.586036Z",
                    "last_modified_by": "test@domain.com",
                    "last_modified_by_type": "User"
                }
        type:
            description:
                - The type of the resource.
            type: str
            returned: always
            sample: 'Microsoft.ContainerRegistry/registries/tokens'
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMContainerRegistryTokenInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True,
            ),
            registry_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.resource_group = None
        self.registry_name = None
        self.name = None

        super(AzureRMContainerRegistryTokenInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec):
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['registry_tokens'] = self.get()
        else:
            self.results['registry_tokens'] = self.list()

        return self.results

    def get(self):
        try:
            response = self.containerregistrytoken_client.tokens.get(resource_group_name=self.resource_group,
                                                                     registry_name=self.registry_name,
                                                                     token_name=self.name)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log("Could not get facts for Registry Token: {0}".format(str(e)))
            return []

        if response:
            return [self.format_item(response)]

    def list(self):
        response = None
        try:
            response = self.containerregistrytoken_client.tokens.list(resource_group_name=self.resource_group,
                                                                      registry_name=self.registry_name)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log("Could not list facts for Registry Token: {0}".format(str(e)))

        return [self.format_item(item) for item in response if response]

    def format_item(self, item):
        result = item.as_dict()
        result['resource_group'] = self.resource_group
        result['registry_name'] = self.registry_name

        return result


def main():
    AzureRMContainerRegistryTokenInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Andrew Klychkov (@Andersson007) <andrew.a.klychkov@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
---
module: clickhouse_db

short_description: Creates or removes a ClickHouse database using the clickhouse-driver Client interface

description:
  - Creates or removes a ClickHouse database using the
    L(clickhouse-driver,https://clickhouse-driver.readthedocs.io/en/latest) Client interface.

version_added: '0.3.0'

attributes:
  check_mode:
    description: Supports check_mode.
    support: full
  idempotent:
    support: partial
    description:
      - Obviously O(state=rename) will fail at second run.

author:
  - Andrew Klychkov (@Andersson007)
  - Aleksandr Vagachev (@aleksvagachev)
  - Rafal Kozlowski (@rkozlo)

extends_documentation_fragment:
  - community.clickhouse.client_inst_opts

options:
  state:
    description:
      - Database state.
      - If C(present), will create the database if not exists.
      - If C(absent), will drop the database if exists.
      - If C(rename), will rename the database if exists.
    type: str
    choices: ['present', 'absent', 'rename']
    default: 'present'
  name:
    description:
      - Database name to add or remove.
    type: str
    required: true
  engine:
    description:
      - Database engine. Once set, cannot be changed.
    type: str
  cluster:
    description:
      - Run the command on all cluster hosts.
      - If the cluster is not configured, the command will crash with an error.
    type: str
    version_added: '0.4.0'
  target:
    description:
      - Name for renaming the database.
    type: str
    version_added: '0.4.0'
  comment:
    description:
      - Database comment.
      - Since C(2.2.0) can change comments in existing databases (requires server 25.8 or later).
    type: str
    version_added: '0.4.0'
'''

EXAMPLES = r'''
- name: Create database
  community.clickhouse.clickhouse_db:
    login_host: localhost
    login_user: alice
    login_db: foo
    login_password: my_password
    name: test_db
    engine: Memory
    state: present
    comment: Test DB

- name: Drop database
  community.clickhouse.clickhouse_db:
    login_host: localhost
    login_user: alice
    login_db: foo
    login_password: my_password
    name: test_db
    state: absent

# Rename the database test_db to prod_db.
# If the database test_db exists, it will be renamed to prod_db.
# If the database test_db does not exist and the prod_db database exists,
# the module will report that nothing has changed.
# If both the databases exist, an error will be raised.
- name: Rename database
  community.clickhouse.clickhouse_db:
    login_host: localhost
    login_user: alice
    login_db: foo
    login_password: my_password
    name: test_db
    target: prod_db
    cluster: test_cluster
    state: rename
'''

RETURN = r'''
executed_statements:
  description:
  - Data-modifying executed statements.
  returned: on success
  type: list
  sample: ['CREATE DATABASE test_db']
'''

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.clickhouse.plugins.module_utils.clickhouse import (
    check_clickhouse_driver,
    client_common_argument_spec,
    connect_to_db_via_client,
    execute_query,
    get_main_conn_kwargs,
    get_server_version,
)


executed_statements = []


class ClickHouseDB():
    def __init__(self, module, client, name, cluster, comment):
        self.module = module
        self.client = client
        self.name = name
        self.cluster = cluster
        # Set default values, then update
        self.exists = False
        self.engine = None
        self.comment = comment
        self.__populate_info()

    def __populate_info(self):
        # TODO: If anyone can determine the version when the comment feature
        # was added to database more precisely, you're welcome to adjust it here
        server_version = get_server_version(self.module, self.client)
        if server_version['year'] >= 22:
            # The comment is not supported in all versions
            query = ("SELECT engine, comment "
                     "FROM system.databases "
                     "WHERE name = %(name)s")
        else:
            query = "SELECT engine FROM system.databases WHERE name = %(name)s"

        # Will move this function to the lib later and reuse
        exec_kwargs = {'params': {'name': self.name}}
        result = execute_query(self.module, self.client, query, exec_kwargs)

        # Assume the DB does not exist by default
        if result:
            # If exists
            self.exists = True
            self.engine = result[0][0]
            if server_version['year'] >= 22:
                self.comment = result[0][1]

    def create(self, engine, comment):
        query = "CREATE DATABASE %s" % self.name
        if self.cluster:
            query += " ON CLUSTER %s" % self.cluster

        if engine:
            query += " ENGINE = %s" % engine

        if comment:
            query += " COMMENT '%s'" % comment

        executed_statements.append(query)

        if not self.module.check_mode:
            execute_query(self.module, self.client, query)

        return True

    def update(self, engine, comment):
        # IMPORTANT: In case in future any items here can change
        # please add check_mode handling

        # There's no way to change the engine
        # so just inform the users they have to recreate
        # the DB in order to change them
        if engine and engine != self.engine:
            msg = ("The provided engine '%s' is different from "
                   "the current one '%s'. It is NOT possible to "
                   "change it. The recreation of the database is required "
                   "in order to change it." % (engine, self.engine))
            self.module.warn(msg)
        server_version = get_server_version(self.module, self.client)
        # At this moment ALTER DATABASE supports only MODIFY COMMENT.
        # When it will support more options probably better
        # will be moving query builder above and here only link comment.
        if comment and comment != self.comment:
            if server_version['year'] < 22:
                msg = ('The module supports the comment feature for ClickHouse '
                       'versions equal to or higher than 22.*. Ignored.')
                self.module.warn(msg)
            elif (server_version['year'] == 25 and server_version['feature'] >= 8) or server_version['year'] >= 26:
                query = "ALTER DATABASE %s" % self.name
                if self.cluster:
                    query += " ON CLUSTER %s" % self.cluster
                query += " MODIFY COMMENT '%s'" % comment

                executed_statements.append(query)

                if not self.module.check_mode:
                    execute_query(self.module, self.client, query)

                return True
            else:
                self.module.warn(
                    f"Server version {server_version['year']}.{server_version['feature']} "
                    f"does not support MODIFY COMMENT. Required: 25.8 or higher"
                )

        return False

    def rename(self, target):
        query = "RENAME DATABASE %s TO %s" % (self.name, target)
        if self.cluster:
            query += " ON CLUSTER %s" % self.cluster

        executed_statements.append(query)

        if not self.module.check_mode:
            execute_query(self.module, self.client, query)

        return True

    def drop(self):
        query = "DROP DATABASE %s" % self.name
        if self.cluster:
            query += " ON CLUSTER %s" % self.cluster

        executed_statements.append(query)

        if not self.module.check_mode:
            execute_query(self.module, self.client, query)

        return True


def main():
    # Set up arguments.
    # If there are common arguments shared across several modules,
    # create the common_argument_spec() function under plugins/module_utils/*
    # and invoke here to return a dict with those arguments
    argument_spec = client_common_argument_spec()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent', 'rename'], default='present'),
        name=dict(type='str', required=True),
        engine=dict(type='str', default=None),
        cluster=dict(type='str', default=None),
        target=dict(type='str', default=None),
        comment=dict(type='str', default=None),
    )

    # Instantiate an object of module class
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    # Assign passed options to variables
    client_kwargs = module.params['client_kwargs']
    # The reason why these arguments are separate from client_kwargs
    # is that we need to protect some sensitive data like passwords passed
    # to the module from logging (see the arguments above with no_log=True);
    # Such data must be passed as module arguments (not nested deep in values).
    main_conn_kwargs = get_main_conn_kwargs(module)
    state = module.params['state']
    name = module.params['name']
    engine = module.params['engine']
    cluster = module.params['cluster']
    target = module.params['target']
    comment = module.params['comment']

    # Will fail if no driver informing the user
    check_clickhouse_driver(module)

    # Connect to DB
    client = connect_to_db_via_client(module, main_conn_kwargs, client_kwargs)

    # Do the job
    changed = False
    database = ClickHouseDB(module, client, name, cluster, comment)

    if state == 'present':
        if not database.exists:
            changed = database.create(engine, comment)
        else:
            # If database exists
            changed = database.update(engine, comment)
    elif state == 'rename':
        if database.exists:
            changed = database.rename(target)
        else:
            target_db = ClickHouseDB(module, client, target, cluster, comment)
            if target_db.exists:
                changed = False
                msg = "There is nothing to rename"
                module.warn(msg)
            else:
                msg = "The %s and %s databases do not exist" % (name, target)
                module.fail_json(msg=msg)
    else:
        # If state is absent
        if database.exists:
            changed = database.drop()

    # Close connection
    client.disconnect_connection()

    # Users will get this in JSON output after execution
    module.exit_json(changed=changed, executed_statements=executed_statements)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = r"""
---
module: update_fact
short_description: Update currently set facts
version_added: "1.0.0"
description:
    - This module allows updating existing variables.
    - Variables are updated on a host-by-host basis.
    - Variables are not modified in place, instead they are returned by the module.
options:
  updates:
    description:
      - A list of dictionaries, each a desired update to make.
    type: list
    elements: dict
    required: True
    suboptions:
      path:
        description:
        - The path in a currently set variable to update.
        - The path can be in dot or bracket notation.
        - It should be a valid jinja reference.
        type: str
        required: True
      value:
        description:
        - The value to be set at the path.
        - Can be a simple or complex data structure.
        type: raw
        required: True


notes:

author:
- Bradley Thornton (@cidrblock)
"""

EXAMPLES = r"""

# Update an existing fact, dot or bracket notation
- name: Set a fact
  ansible.builtin.set_fact:
    a:
      b:
        c:
          - 1
          - 2

- name: Update the fact
  ansible.utils.update_fact:
    updates:
      - path: a.b.c.0
        value: 10
      - path: "a['b']['c'][1]"
        value: 20
  register: updated

- debug:
    var: updated.a

# updated:
#   a:
#     b:
#       c:
#       - 10
#       - 20
#   changed: true


# Lists can be appended, new keys added to dictionaries

- name: Set a fact
  ansible.builtin.set_fact:
    a:
      b:
        b1:
          - 1
          - 2

- name: Update, add to list, add new key
  ansible.utils.update_fact:
    updates:
      - path: a.b.b1.2
        value: 3
      - path: a.b.b2
        value:
          - 10
          - 20
          - 30
  register: updated

- debug:
    var: updated.a

# updated:
#   a:
#     b:
#       b1:
#       - 1
#       - 2
#       - 3
#       b2:
#       - 10
#       - 20
#       - 30
#   changed: true

#####################################################################
# Update every item in a list of dictionaries
# build the update list ahead of time using a loop
# and then apply the changes to the fact
#####################################################################

- name: Set fact
  ansible.builtin.set_fact:
    addresses:
      - raw: 10.1.1.0/255.255.255.0
        name: servers
      - raw: 192.168.1.0/255.255.255.0
        name: printers
      - raw: 8.8.8.8
        name: dns

- name: Build a list of updates
  ansible.builtin.set_fact:
    update_list: "{{ update_list + update }}"
  loop: "{{ addresses }}"
  loop_control:
    index_var: idx
  vars:
    update_list: []
    update:
      - path: addresses[{{ idx }}].network
        value: "{{ item['raw'] | ansible.utils.ipaddr('network') }}"
      - path: addresses[{{ idx }}].prefix
        value: "{{ item['raw'] | ansible.utils.ipaddr('prefix') }}"

- debug:
    var: update_list

# TASK [debug] *******************
# ok: [localhost] =>
#   update_list:
#   - path: addresses[0].network
#     value: 10.1.1.0
#   - path: addresses[0].prefix
#     value: '24'
#   - path: addresses[1].network
#     value: 192.168.1.0
#   - path: addresses[1].prefix
#     value: '24'
#   - path: addresses[2].network
#     value: 8.8.8.8
#   - path: addresses[2].prefix
#     value: '32'

- name: Make the updates
  ansible.utils.update_fact:
    updates: "{{ update_list }}"
  register: updated

- debug:
    var: updated

# TASK [debug] ***********************
# ok: [localhost] =>
#   updated:
#     addresses:
#     - name: servers
#       network: 10.1.1.0
#       prefix: '24'
#       raw: 10.1.1.0/255.255.255.0
#     - name: printers
#       network: 192.168.1.0
#       prefix: '24'
#       raw: 192.168.1.0/255.255.255.0
#     - name: dns
#       network: 8.8.8.8
#       prefix: '32'
#       raw: 8.8.8.8
#     changed: true
#     failed: false


#####################################################################
# Retrieve, update, and apply interface description change
# use index_of to locate Etherent1/1
#####################################################################

- name: Get the current interface config
  cisco.nxos.nxos_interfaces:
    state: gathered
  register: interfaces

- name: Update the description of Ethernet1/1
  ansible.utils.update_fact:
    updates:
      - path: "interfaces.gathered[{{ index }}].description"
        value: "Configured by ansible"
  vars:
    index: "{{ interfaces.gathered|ansible.utils.index_of('eq', 'Ethernet1/1', 'name') }}"
  register: updated

- name: Update the configuration
  cisco.nxos.nxos_interfaces:
    config: "{{ updated.interfaces.gathered }}"
    state: overridden
  register: result

- name: Show the commands issued
  debug:
    msg: "{{ result['commands'] }}"

# TASK [Show the commands issued] *************************************
# ok: [nxos101] => {
#     "msg": [
#         "interface Ethernet1/1",
#         "description Configured by ansible"
#     ]
# }


#####################################################################
# Retrieve, update, and apply an ipv4 ACL change
# finding the index of AFI ipv4 acls
# finding the index of the ACL named 'test1'
# finding the index of sequence 10
#####################################################################

- name: Retrieve the current acls
  arista.eos.eos_acls:
    state: gathered
  register: current

- name: Update the source of sequence 10 in the IPv4 ACL named test1
  ansible.utils.update_fact:
    updates:
      - path: current.gathered[{{ afi }}].acls[{{ acl }}].aces[{{ ace }}].source
        value:
          subnet_address: "192.168.2.0/24"
  vars:
    afi: "{{ current.gathered|ansible.utils.index_of('eq', 'ipv4', 'afi') }}"
    acl: "{{ current.gathered[afi|int].acls|ansible.utils.index_of('eq', 'test1', 'name') }}"
    ace: "{{ current.gathered[afi|int].acls[acl|int].aces|ansible.utils.index_of('eq', 10, 'sequence') }}"
  register: updated

- name: Apply the changes
  arista.eos.eos_acls:
    config: "{{ updated.current.gathered }}"
    state: overridden
  register: changes

- name: Show the commands issued
  debug:
    msg: "{{ changes['commands'] }}"

# TASK [Show the commands issued] *************************************
# ok: [eos101] => {
#     "msg": [
#         "ip access-list test1",
#         "no 10",
#         "10 permit ip 192.168.2.0/24 host 10.1.1.2"
#     ]
# }


#####################################################################
# Disable ip redirects on any layer3 interface
# find the layer 3 interfaces
# use each name to find their index in l3 interface
# build an 'update' list and apply the updates
#####################################################################

- name: Get the current interface and L3 interface configuration
  cisco.nxos.nxos_facts:
    gather_subset: min
    gather_network_resources:
      - interfaces
      - l3_interfaces

- name: Build the list of updates to make
  ansible.builtin.set_fact:
    updates: "{{ updates + [entry] }}"
  vars:
    updates: []
    entry:
      path: "ansible_network_resources.l3_interfaces[{{ item }}].redirects"
      value: false
    w_mode: "{{ ansible_network_resources.interfaces|selectattr('mode', 'defined') }}"
    m_l3: "{{ w_mode|selectattr('mode', 'eq', 'layer3') }}"
    names: "{{ m_l3|map(attribute='name')|list }}"
    l3_indicies: "{{ ansible_network_resources.l3_interfaces|ansible.utils.index_of('in', names, 'name', wantlist=True) }}"
  loop: "{{ l3_indicies }}"

# TASK [Build the list of updates to make] ****************************
# ok: [nxos101] => (item=99) => changed=false
#   ansible_facts:
#     updates:
#     - path: ansible_network_resources.l3_interfaces[99].redirects
#       value: false
#   ansible_loop_var: item
#   item: 99

- name: Update the l3 interfaces
  ansible.utils.update_fact:
    updates: "{{ updates }}"
  register: updated

# TASK [Update the l3 interfaces] *************************************
# changed: [nxos101] => changed=true
#   ansible_network_resources:
#     l3_interfaces:
#     <...>
#     - ipv4:
#       - address: 10.1.1.1/24
#       name: Ethernet1/100
#       redirects: false

- name: Apply the configuration changes
  cisco.nxos.l3_interfaces:
    config: "{{ updated.ansible_network_resources.l3_interfaces }}"
    state: overridden
  register: changes

# TASK [Apply the configuration changes] ******************************
# changed: [nxos101] => changed=true
#   commands:
#   - interface Ethernet1/100
#   - no ip redirects
"""



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos_static_routes config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""


from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.static_routes import (
    Static_routesTemplate,
)


class Static_routes(ResourceModule):
    """
    The nxos_static_routes config class
    """

    def __init__(self, module):
        super(Static_routes, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="static_routes",
            tmplt=Static_routesTemplate(),
        )

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd, delete_spcl = self.list_to_dict(self.want, "want")
        haved, n_req = self.list_to_dict(self.have, "have")

        if delete_spcl and haved and self.state == "deleted":
            for pk, to_rem in delete_spcl.items():
                if pk in ["ipv4", "ipv6"]:
                    _afis = haved.get("(_afis_)")
                    for k, v in _afis.get(pk, {}).items():
                        for each_dest in to_rem:
                            if k.split("_")[0] == each_dest:
                                self.addcmd({pk: v}, pk, True)
                else:
                    sp_begin = len(self.commands)
                    _vrfs = haved.get(pk)
                    for ak, v in _vrfs.items():
                        for k, srts in v.items():
                            for each_dest in to_rem.get(ak):
                                if k.split("_")[0] == each_dest:
                                    self.addcmd({ak: srts}, ak, True)
                    if len(self.commands) != sp_begin:
                        self.commands.insert(
                            sp_begin,
                            self._tmplt.render({"namevrf": pk}, "vrf", False),
                        )

        else:
            # if state is merged, merge want onto have and then compare
            if self.state == "merged":
                wantd = dict_merge(haved, wantd)

            for k, want in wantd.items():
                self._compare_top_level_keys(want=want, have=haved.pop(k, {}), vrf_name=k)

            if (self.state == "deleted" and not wantd) or self.state in ["overridden"]:
                for k, have in haved.items():
                    self._compare_top_level_keys(want={}, have=have, vrf_name=k)

    def _compare_top_level_keys(self, want, have, vrf_name):
        begin = len(self.commands)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted" and have:
            _have = {}
            for addf in ["ipv4", "ipv6"]:
                _temp_sr = {}
                for k, ha in have.get(addf, {}).items():
                    if k in want.get(addf, {}):  # or not want.get(addf)
                        _temp_sr[k] = ha
                    if _temp_sr:
                        _have[addf] = _temp_sr
            if _have:
                have = _have
                want = {}

        if self.state != "deleted":
            for _afi, routes in want.items():
                self._compare(s_want=routes, s_have=have.pop(_afi, {}), afi=_afi)

        if self.state in ["overridden", "deleted"]:
            for _afi, routes in have.items():
                self._compare(s_want={}, s_have=routes, afi=_afi)

        if len(self.commands) != begin:
            if vrf_name == "(_afis_)":
                afi_cmds = []
                for cmds in range(begin, len(self.commands)):
                    self.commands.insert(0, self.commands.pop())
                #     afi_cmds.append(self.commands.pop())
                # self.commands = afi_cmds + self.commands
            else:
                self.commands.insert(begin, self._tmplt.render({"namevrf": vrf_name}, "vrf", False))

    def _compare(self, s_want, s_have, afi):
        for name, w_srs in s_want.items():
            have_srs = s_have.pop(name, {})
            self.compare(parsers=afi, want={afi: w_srs}, have={afi: have_srs})

        # remove remaining items in have for replaced state
        for name, h_srs in s_have.items():
            self.compare(parsers=afi, want={}, have={afi: h_srs})

    def list_to_dict(self, param, operation):
        _static_rts = {}
        _delete_spc = {}
        if param:
            for srs in param:
                _vrf = srs.get("vrf")
                _srts = {}
                for adfs in srs.get("address_families", []):
                    _afi = adfs.get("afi")
                    _routes = {}
                    for rts in adfs.get("routes", []):
                        _dest = rts.get("dest", "")

                        #  below if specific to special delete
                        if (
                            self.state == "deleted"
                            and operation == "want"
                            and not rts.get("next_hops")
                        ):
                            if _vrf:
                                if not _delete_spc.get(_vrf):
                                    _delete_spc[_vrf] = {}
                                if not _delete_spc[_vrf].get(_afi):
                                    _delete_spc[_vrf][_afi] = []
                                _delete_spc[_vrf][_afi].append(_dest)
                            else:
                                if not _delete_spc.get(_afi):
                                    _delete_spc[_afi] = []
                                _delete_spc[_afi].append(_dest)

                        for nxh in rts.get("next_hops", []):
                            _forw_rtr_add = nxh.get("forward_router_address", "").upper()
                            _intf = nxh.get("interface", "")
                            _rt_name = nxh.get("route_name", "")
                            _dest_vrf = nxh.get("dest_vrf", "")
                            _key = _dest + "_" + _forw_rtr_add + _intf + _rt_name + "_" + _dest_vrf

                            dummy_sr = {
                                "afi": _afi,
                                "dest": _dest,
                            }

                            if _vrf:
                                dummy_sr["vrf"] = _vrf
                            if _intf:
                                dummy_sr["interface"] = _intf
                            if _forw_rtr_add:
                                dummy_sr["forward_router_address"] = _forw_rtr_add
                            if _rt_name:
                                dummy_sr["route_name"] = _rt_name
                            dummy_sr.update(nxh)

                            _routes[_key] = dummy_sr
                    _srts[_afi] = _routes
                _static_rts[_vrf if _vrf else "(_afis_)"] = _srts
        return _static_rts, _delete_spc



# Copyright (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import base64
import datetime
import os
import re
import textwrap
import traceback
import typing as t
from collections.abc import Callable
from urllib.parse import unquote

from ansible_collections.community.crypto.plugins.module_utils._acme.errors import (
    ModuleFailException,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.math import (
    convert_int_to_bytes,
)
from ansible_collections.community.crypto.plugins.module_utils._time import (
    get_now_datetime,
)

if t.TYPE_CHECKING:
    from ansible_collections.community.crypto.plugins.module_utils._acme.backends import (  # pragma: no cover
        CertificateInformation,
        CryptoBackend,
    )


def nopad_b64(data: bytes) -> str:
    return base64.urlsafe_b64encode(data).decode("utf8").replace("=", "")


def der_to_pem(der_cert: bytes) -> str:
    """
    Convert the DER format certificate in der_cert to a PEM format certificate and return it.
    """
    content = "\n".join(textwrap.wrap(base64.b64encode(der_cert).decode("utf8"), 64))
    return f"-----BEGIN CERTIFICATE-----\n{content}\n-----END CERTIFICATE-----\n"


def pem_to_der(
    *, pem_filename: str | os.PathLike | None = None, pem_content: str | None = None
) -> bytes:
    """
    Load PEM file, or use PEM file's content, and convert to DER.

    If PEM contains multiple entities, the first entity will be used.
    """
    certificate_lines = []
    if pem_content is not None:
        lines = pem_content.splitlines()
    elif pem_filename is not None:
        try:
            with open(pem_filename, "r", encoding="utf-8") as f:
                lines = list(f)
        except Exception as err:
            raise ModuleFailException(
                f"cannot load PEM file {pem_filename}: {err}",
                exception=traceback.format_exc(),
            ) from err
    else:
        raise ModuleFailException(
            "One of pem_filename and pem_content must be provided"
        )
    header_line_count = 0
    for line in lines:
        if line.startswith("-----"):
            header_line_count += 1
            if header_line_count == 2:
                # If certificate file contains other certs appended
                # (like intermediate certificates), ignore these.
                break
            continue
        certificate_lines.append(line.strip())
    return base64.b64decode("".join(certificate_lines))


def process_links(
    *, info: dict[str, t.Any], callback: Callable[[str, str], None]
) -> None:
    """
    Process link header, calls callback for every link header with the URL and relation as options.

    https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Link
    """
    if "link" in info:
        link = info["link"]
        for url, relation in re.findall(r'<([^>]+)>;\s*rel="(\w+)"', link):
            callback(unquote(url), relation)


def parse_retry_after(
    value: str,
    *,
    relative_with_timezone: bool = True,
    now: datetime.datetime | None = None,
) -> datetime.datetime:
    """
    Parse the value of a Retry-After header and return a timestamp.

    https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Retry-After
    """
    # First try a number of seconds
    try:
        delta = datetime.timedelta(seconds=int(value))
        if now is None:
            now = get_now_datetime(with_timezone=relative_with_timezone)
        return now + delta
    except ValueError:
        pass

    try:
        return datetime.datetime.strptime(value, "%a, %d %b %Y %H:%M:%S GMT")
    except ValueError:
        pass

    raise ValueError(f"Cannot parse Retry-After header value {repr(value)}")


def compute_cert_id(
    *,
    backend: CryptoBackend,
    cert_info: CertificateInformation | None = None,
    cert_filename: str | os.PathLike | None = None,
    cert_content: str | bytes | None = None,
    none_if_required_information_is_missing: bool = False,
) -> str | None:
    # Obtain certificate info if not provided
    if cert_info is None:
        cert_info = backend.get_cert_information(
            cert_filename=cert_filename, cert_content=cert_content
        )

    # Convert Authority Key Identifier to string
    if cert_info.authority_key_identifier is None:
        if none_if_required_information_is_missing:
            return None
        raise ModuleFailException(
            "Certificate has no Authority Key Identifier extension"
        )
    aki = (
        (base64.urlsafe_b64encode(cert_info.authority_key_identifier))
        .decode("ascii")
        .replace("=", "")
    )

    # Convert serial number to string
    serial_bytes = convert_int_to_bytes(cert_info.serial_number)
    if ord(serial_bytes[:1]) >= 128:
        serial_bytes = b"\x00" + serial_bytes
    serial = (base64.urlsafe_b64encode(serial_bytes)).decode("ascii").replace("=", "")

    # Compose cert ID
    return f"{aki}.{serial}"


__all__ = (
    "compute_cert_id",
    "der_to_pem",
    "nopad_b64",
    "parse_retry_after",
    "pem_to_der",
    "process_links",
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_bios_ai_tuning
short_description: AI-optimized BIOS policy presets for Cisco Intersight
description:
  - Creates BIOS policies with pre-validated settings optimized for AI and GPU workloads.
  - Provides named tuning profiles that set multiple BIOS knobs at once for common AI use cases.
  - Profiles target NVIDIA H100/B200 GPU servers and Intel/AMD CPU platforms used in Cisco AI Factory.
  - Individual BIOS knobs can be overridden after the preset is applied.
  - For full BIOS knob control, use M(cisco.intersight.intersight_bios_policy) instead.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the BIOS policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the BIOS policy.
    type: str
    aliases: [descr]
  tuning_profile:
    description:
      - The AI tuning profile preset to apply.
      - C(gpu_inference) optimizes for maximum GPU throughput with low-latency memory access.
        Sets GPU ACS control enabled, NUMA optimized, hardware P-states disabled for consistent
        performance, SR-IOV enabled, and memory interleave optimized.
      - C(gpu_training) optimizes for large-scale GPU training workloads with maximum memory
        bandwidth. Includes all gpu_inference settings plus maximum power performance tuning,
        workload configuration set to balanced, and package C-state limit for sustained throughput.
      - C(cpu_inference) optimizes for CPU-based inference with low-latency tuning.
        Enables hardware P-states for native performance, Intel Turbo Boost, LLC prefetch,
        and NUMA optimization without GPU-specific settings.
      - C(edge_ai) optimizes for Cisco Unified Edge nodes with power-aware AI tuning.
        Balances performance with power efficiency using energy-efficient turbo, hardware P-states,
        and NUMA optimization suitable for edge deployment constraints.
    type: str
    choices: [gpu_inference, gpu_training, cpu_inference, edge_ai]
  gpu_count:
    description:
      - Number of GPUs to enable via ACS Control (1-8).
      - Only applicable for gpu_inference and gpu_training profiles.
      - GPUs beyond this count will have ACS control set to platform-default.
    type: int
    default: 8
  numa_optimized:
    description:
      - Override the NUMA optimization setting from the tuning profile.
    type: str
    choices: [platform-default, enabled, disabled]
  cpu_power_management:
    description:
      - Override the CPU power management setting from the tuning profile.
    type: str
    choices: [platform-default, performance, energy-efficient, custom]
  sriov:
    description:
      - Override the SR-IOV setting from the tuning profile.
      - SR-IOV is required for GPU passthrough in virtualized environments.
    type: str
    choices: [platform-default, enabled, disabled]
  intel_turbo_boost_tech:
    description:
      - Override the Intel Turbo Boost Technology setting.
    type: str
    choices: [platform-default, enabled, disabled]
  cpu_performance:
    description:
      - Override the CPU performance setting from the tuning profile.
    type: str
    choices: [platform-default, custom, enterprise, high-throughput, hpc]
  cpu_energy_performance:
    description:
      - Override the CPU energy performance bias setting.
    type: str
    choices: [platform-default, balanced-energy, balanced-performance, energy-efficient, performance]
author:
  - Steve Fulmer (@stevefulme1)
'''

EXAMPLES = r'''
- name: Create GPU inference optimized BIOS policy for 8-GPU server
  cisco.intersight.intersight_bios_ai_tuning:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: AI-Factory
    name: bios-gpu-inference-h100
    description: BIOS tuning for H100 GPU inference servers
    tuning_profile: gpu_inference
    gpu_count: 8

- name: Create GPU training BIOS policy with power override
  cisco.intersight.intersight_bios_ai_tuning:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: bios-gpu-training
    tuning_profile: gpu_training
    cpu_power_management: performance

- name: Create CPU inference BIOS policy for non-GPU workloads
  cisco.intersight.intersight_bios_ai_tuning:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: bios-cpu-inference
    tuning_profile: cpu_inference
    intel_turbo_boost_tech: enabled

- name: Create edge AI BIOS policy for Unified Edge nodes
  cisco.intersight.intersight_bios_ai_tuning:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: bios-edge-ai
    tuning_profile: edge_ai
    gpu_count: 2

- name: Delete AI BIOS policy
  cisco.intersight.intersight_bios_ai_tuning:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: bios-gpu-inference-h100
    tuning_profile: gpu_inference
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "bios-gpu-inference-h100",
        "ObjectType": "bios.Policy",
        "NumaOptimized": "enabled",
        "AcsControlGpu1state": "enabled"
    }
applied_settings:
  description: The BIOS settings applied by the tuning profile and any overrides.
  returned: when state is present
  type: dict
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


# AI tuning profile presets
# Each profile maps Intersight BIOS API property names to their values.
TUNING_PROFILES = {
    'gpu_inference': {
        # NUMA and memory optimization
        'NumaOptimized': 'enabled',
        'CbsDfCmnAcpiSratL3numa': 'enabled',
        # CPU performance - disable P-states for consistent GPU-driven latency
        'CpuPowerManagement': 'performance',
        'CpuPerformance': 'high-throughput',
        'CpuEnergyPerformance': 'performance',
        'PackageCstateLimit': 'C0 C1 State',
        'AutonomousCstateEnable': 'disabled',
        'CpuHwpm': 'Disabled',
        # GPU and I/O
        'SrIov': 'enabled',
        'AcsControlSlot11state': 'enabled',
        'AcsControlSlot12state': 'enabled',
        'AcsControlSlot13state': 'enabled',
        'AcsControlSlot14state': 'enabled',
        # Memory performance
        'LlcPrefetch': 'enabled',
        'XptPrefetch': 'enabled',
    },
    'gpu_training': {
        # All gpu_inference settings plus maximum power
        'NumaOptimized': 'enabled',
        'CbsDfCmnAcpiSratL3numa': 'enabled',
        'CpuPowerManagement': 'performance',
        'CpuPerformance': 'high-throughput',
        'CpuEnergyPerformance': 'performance',
        'PackageCstateLimit': 'C0 C1 State',
        'AutonomousCstateEnable': 'disabled',
        'CpuHwpm': 'Disabled',
        'SrIov': 'enabled',
        'AcsControlSlot11state': 'enabled',
        'AcsControlSlot12state': 'enabled',
        'AcsControlSlot13state': 'enabled',
        'AcsControlSlot14state': 'enabled',
        'LlcPrefetch': 'enabled',
        'XptPrefetch': 'enabled',
        # Training-specific: maximum power and bandwidth
        'WorkloadConfig': 'Balanced',
    },
    'cpu_inference': {
        # CPU-optimized for inference without GPU focus
        'NumaOptimized': 'enabled',
        'CbsDfCmnAcpiSratL3numa': 'enabled',
        'CpuPowerManagement': 'performance',
        'CpuPerformance': 'high-throughput',
        'CpuEnergyPerformance': 'performance',
        'IntelTurboBoostTech': 'enabled',
        'CpuHwpm': 'HWPM Native Mode',
        'LlcPrefetch': 'enabled',
        'XptPrefetch': 'enabled',
    },
    'edge_ai': {
        # Power-aware AI tuning for edge nodes
        'NumaOptimized': 'enabled',
        'CpuPowerManagement': 'energy-efficient',
        'CpuPerformance': 'enterprise',
        'CpuEnergyPerformance': 'balanced-performance',
        'EnergyEfficientTurbo': 'enabled',
        'IntelTurboBoostTech': 'enabled',
        'CpuHwpm': 'HWPM Native Mode',
        'LlcPrefetch': 'enabled',
    },
}

# GPU ACS control API property names by slot index (1-8)
GPU_ACS_PROPERTIES = [
    'AcsControlGpu1state',
    'AcsControlGpu2state',
    'AcsControlGpu3state',
    'AcsControlGpu4state',
    'AcsControlGpu5state',
    'AcsControlGpu6state',
    'AcsControlGpu7state',
    'AcsControlGpu8state',
]

# Map of override parameter names to API property names
OVERRIDE_MAP = {
    'numa_optimized': 'NumaOptimized',
    'cpu_power_management': 'CpuPowerManagement',
    'sriov': 'SrIov',
    'intel_turbo_boost_tech': 'IntelTurboBoostTech',
    'cpu_performance': 'CpuPerformance',
    'cpu_energy_performance': 'CpuEnergyPerformance',
}


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        tuning_profile=dict(type='str', choices=['gpu_inference', 'gpu_training', 'cpu_inference', 'edge_ai']),
        gpu_count=dict(type='int', default=8),
        numa_optimized=dict(type='str', choices=['platform-default', 'enabled', 'disabled']),
        cpu_power_management=dict(type='str', choices=['platform-default', 'performance', 'energy-efficient', 'custom']),
        sriov=dict(type='str', choices=['platform-default', 'enabled', 'disabled']),
        intel_turbo_boost_tech=dict(type='str', choices=['platform-default', 'enabled', 'disabled']),
        cpu_performance=dict(type='str', choices=['platform-default', 'custom', 'enterprise', 'high-throughput', 'hpc']),
        cpu_energy_performance=dict(
            type='str',
            choices=['platform-default', 'balanced-energy', 'balanced-performance', 'energy-efficient', 'performance']
        ),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/bios/Policies'

    intersight.api_body = {
        'Name': intersight.module.params['name'],
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
    }

    if module.params['state'] == 'present':
        if not module.params.get('tuning_profile'):
            module.fail_json(msg="tuning_profile is required when state is 'present'")

        intersight.set_tags_and_description()

        # Apply tuning profile preset
        profile_name = module.params['tuning_profile']
        profile_settings = TUNING_PROFILES[profile_name].copy()

        # Apply GPU ACS control based on gpu_count for GPU profiles
        if profile_name in ('gpu_inference', 'gpu_training'):
            gpu_count = min(max(module.params['gpu_count'], 0), 8)
            for i, prop in enumerate(GPU_ACS_PROPERTIES):
                profile_settings[prop] = 'enabled' if i < gpu_count else 'platform-default'

        # Apply user overrides
        for param_name, api_property in OVERRIDE_MAP.items():
            if module.params.get(param_name) is not None:
                profile_settings[api_property] = module.params[param_name]

        intersight.api_body.update(profile_settings)
        intersight.result['applied_settings'] = profile_settings

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_sqlmidatabase_info
version_added: "2.4.0"
short_description: Get Azure SQL managed instance database facts
description:
    - Get facts of Azure SQL managed instance database facts.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource.
        type: str
        required: true
    managed_instance_name:
        description:
            - The name of the SQL managed instance.
        type: str
        required: true
    database_name:
        description:
            - The name of the SQL managed instance database.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get SQL managed instance database by name
  azure_rm_sqlmidatabase_info:
    resource_group: testrg
    managed_instance_name: testinstancename
    database_name: newdatabase
'''

RETURN = '''
database:
    description:
        - A dictionary containing facts for SQL Managed Instance database info.
    returned: always
    type: complex
    contains:
        auto_complete_restore:
            description:
                - Whether to auto complete restore of this managed database.
            type: bool
            returned: always
            sample: null
        catalog_collation:
            description:
                - Collation of the metadata catalog.
            type: str
            returned: always
            sample: null
        create_mode:
            description:
                - Managed database create mode.
            type: str
            returned: always
            sample: null
        create_date:
            description:
                - Creation date of the database.
            type: str
            returned: always
            sample: "2024-05-06T23:59:49.770Z"
        database_name:
            description:
                - The sql mi databse name.
            type: str
            returned: always
            sample: fredtest
        default_secondary_location:
            description:
                - Geo paired region.
            type: str
            returned: always
            sample: westus
        id:
            description:
                - The resource ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxxx/resourceGroups/testRG/providers/Microsoft.Sql/managedInstances/fredsqlmin/databases/fredtest"
        last_backup_name:
            description:
                - Last backup file name for restore of this managed database.
            type: str
            returned: always
            sample: null
        location:
            description:
                - The resource's location.
            type: str
            returned: always
            sample: eastus
        long_term_retention_backup_resource_id:
            description:
                - The name of the Long Term Retention backup to be used for restore of this managed database.
            type: str
            returned: always
            sample: null
        managed_instance_name:
            description:
                - The name of the SQL managed instance.
            type: str
            returned: always
            sample: fredsqlmin
        recoverable_database_id:
            description:
                - The resource identifier of the recoverable database associated with the database.
            type: str
            returned: always
            sample: null
        resource_group:
            description:
                - The resource's resource group.
            type: str
            returned: always
            sample: testRG
        restorable_dropped_database_id:
            description:
                - The restorable dropped database resource id.
            type: str
            returned: always
            sample: null
        restore_point_in_time:
            description:
                - Specifies the point in time (ISO8601 format) of the source database.
            type: str
            returned: always
            sample: null
        source_database_id:
            description:
                - The resource identifier of the source database associated with create operation of this database.
            type: str
            returned: always
            sample: null
        status:
            description:
                - Status of the database.
            type: str
            returned: always
            sample: online
        storage_container_sas_token:
            description:
                - Specifies the storage container sas token.
            type: str
            returned: always
            sample: null
        storage_container_uri:
            description:
                - Specifies the uri of the storage container where backups for this restore are stopped.
            type: str
            returned: always
            sample: null
        tags:
            description:
                - The resource's tags
            type: str
            returned: always
            sample: {key1: value1}
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: "Microsoft.Sql/managedInstances/databases"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import HttpResponseError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMSqlMIDatabaseInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True,
            ),
            managed_instance_name=dict(
                type='str',
                required=True,
            ),
            database_name=dict(
                type='str',
            ),
            tags=dict(
                type='list',
                elements='str'
            ),
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.resource_group = None
        self.managed_instance_name = None
        self.database_name = None
        self.tags = None

        super(AzureRMSqlMIDatabaseInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.database_name is not None:
            self.results['database'] = self.get()
        else:
            self.results['database'] = self.list_by_instance()
        return self.results

    def list_by_instance(self):
        response = None
        results = []
        try:
            response = self.sql_client.managed_databases.list_by_instance(resource_group_name=self.resource_group,
                                                                          managed_instance_name=self.managed_instance_name)
            self.log("Response : {0}".format(response))
        except HttpResponseError:
            self.log('Could not get facts for SQL managed instance database.')

        if response is not None:
            for item in response:
                if self.has_tags(item.tags, self.tags):
                    results.append(self.format_item(item))
        return results

    def get(self):
        response = None
        try:
            response = self.sql_client.managed_databases.get(resource_group_name=self.resource_group,
                                                             managed_instance_name=self.managed_instance_name,
                                                             database_name=self.database_name)
            self.log("Response : {0}".format(response))
        except HttpResponseError as ec:
            self.log('Could not get facts for SQL managed instance database.')

        if response is not None and self.has_tags(response.tags, self.tags):
            return [self.format_item(response)]

    def format_item(self, item):
        d = item.as_dict()
        d = {
            'resource_group': self.resource_group,
            'managed_instance_name': self.managed_instance_name,
            'database_name': d.get('name'),
            'id': d.get('id', None),
            'type': d.get('type', None),
            'location': d.get('location'),
            'tags': d.get('tags'),
            'collation': d.get('collation'),
            'status': d.get('status'),
            'creation_date': d.get('creation_date'),
            'earliest_restore_point': d.get('earliest_restore_point'),
            'restore_point_in_time': d.get('restore_point_in_time'),
            'default_secondary_location': d.get('default_secondary_location'),
            'catalog_collation': d.get('catalog_collation'),
            'create_mode': d.get('create_mode'),
            'storage_container_uri': d.get('storage_container_uri'),
            'source_database_id': d.get('source_database_id'),
            'restorable_dropped_database_id': d.get('restorable_dropped_database_id'),
            'storage_container_sas_token': d.get('storage_container_sas_token'),
            'failover_group_id': d.get('failover_group_id'),
            'recoverable_database_id': d.get('recoverable_database_id'),
            'long_term_retention_backup_resource_id': d.get('long_term_retention_backup_resource_id'),
            'auto_complete_restore': d.get('auto_complete_restore'),
            'last_backup_name': d.get('last_backup_name')
        }
        return d


def main():
    AzureRMSqlMIDatabaseInfo()


if __name__ == '__main__':
    main()



# Copyright (c) 2017, Brian Coca
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import annotations

DOCUMENTATION = r"""
name: yaml
short_description: YAML formatted files
description:
  - This cache uses YAML formatted, per host, files saved to the filesystem.
author: Brian Coca (@bcoca)
options:
  _uri:
    required: true
    description:
      - Path in which the cache plugin saves the files.
    env:
      - name: ANSIBLE_CACHE_PLUGIN_CONNECTION
    ini:
      - key: fact_caching_connection
        section: defaults
    type: string
  _prefix:
    description: User defined prefix to use when creating the files.
    env:
      - name: ANSIBLE_CACHE_PLUGIN_PREFIX
    ini:
      - key: fact_caching_prefix
        section: defaults
    type: string
  _timeout:
    default: 86400
    description: Expiration timeout in seconds for the cache plugin data. Set to 0 to never expire.
    env:
      - name: ANSIBLE_CACHE_PLUGIN_TIMEOUT
    ini:
      - key: fact_caching_timeout
        section: defaults
    type: integer
        # TODO: determine whether it is OK to change to: type: float
"""

import os

import yaml
from ansible.parsing.yaml.dumper import AnsibleDumper
from ansible.parsing.yaml.loader import AnsibleLoader
from ansible.plugins.cache import BaseFileCacheModule


class CacheModule(BaseFileCacheModule):
    """
    A caching module backed by yaml files.
    """

    def _load(self, filepath):
        with open(os.path.abspath(filepath), encoding="utf-8") as f:
            return AnsibleLoader(f).get_single_data()

    def _dump(self, value, filepath):
        with open(os.path.abspath(filepath), "w", encoding="utf-8") as f:
            yaml.dump(value, f, Dumper=AnsibleDumper, default_flow_style=False)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_vrf_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
---
module: ios_vrf_interfaces
extends_documentation_fragment:
  - cisco.ios.ios
short_description: Manages VRF configuration on interfaces.
description:
  - Manages Virtual Routing and Forwarding (VRF) configuration on interfaces of Cisco IOS devices.
version_added: "1.0.0"
author: "AAYUSH ANAND (@AAYUSH2091)"
notes:
  - Tested against Cisco IOS XE Version 17.13.01a
  - VRF must exist before assigning to an interface
  - When removing VRF from interface, associated IP addresses will be removed
options:
  config:
    description: A list of interface VRF configurations.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Full name of the interface to be configured.
          - Example - GigabitEthernet1, Loopback24
        type: str
        required: true
      vrf_name:
        description:
          - Name of the VRF to be configured on the interface.
          - When configured, applies 'vrf forwarding <vrf_name>' under the interface.
        type: str
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by executing
        the command B(show running-config | section ^interface).
      - The state I(parsed) reads the configuration from C(running_config) option and transforms
        it into Ansible structured data as per the resource module's argspec and the value
        is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
      - The state the configuration should be left in.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - gathered
      - rendered
      - parsed
    default: merged
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

- name: Merge provided configuration with device configuration
  cisco.ios.ios_vrf_interfaces:
    config:
      - name: GigabitEthernet1
      - name: GigabitEthernet2
        vrf_name: vrf_D
      - name: GigabitEthernet3
      - name: GigabitEthernet4
    state: merged

# Task Output:
# ------------
#
# before:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#   - name: "GigabitEthernet2"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"
#
# commands:
#   - interface GigabitEthernet2
#   - vrf forwarding vrf_D
#
# after:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#   - name: "GigabitEthernet2"
#     vrf_name: "vrf_D"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"

# After state:
# ------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  vrf forwarding vrf_D
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

# Using overridden

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  vrf forwarding vrf_B
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

- name: Override device configuration with provided configuration
  cisco.ios.ios_vrf_interfaces:
    config:
      - name: GigabitEthernet1
      - name: GigabitEthernet2
      - name: GigabitEthernet3
      - name: GigabitEthernet4
    state: overridden

# Task Output:
# ------------
#
# before:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#   - name: "GigabitEthernet2"
#     vrf_name: "vrf_B"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"
#
# commands:
#   - interface GigabitEthernet2
#   - no vrf forwarding vrf_B
#
# after:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#   - name: "GigabitEthernet2"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"

# After state:
# ------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

# Using gathered

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  vrf forwarding vrf_B
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  no ip address
#  shutdown
#  negotiation auto

- name: Gather listed VRF interfaces
  cisco.ios.ios_vrf_interfaces:
    state: gathered

# Task Output:
# ------------
#
# gathered:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#   - name: "GigabitEthernet2"
#     vrf_name: "vrf_B"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"

# Using rendered

- name: Render VRF configuration
  cisco.ios.ios_vrf_interfaces:
    config:
      - name: GigabitEthernet1
      - name: GigabitEthernet2
        vrf_name: vrf_D
      - name: GigabitEthernet3
      - name: GigabitEthernet4
    state: rendered

# Task Output:
# ------------
#
# rendered:
#   - interface GigabitEthernet2
#   - vrf forwarding vrf_D

# Using parsed

# File: parsed.cfg
# ---------------
#
# interface GigabitEthernet1
#  vrf vrf_C
#  shutdown
# !
# interface GigabitEthernet2
#  vrf vrf_D
#  shutdown
# !
# interface GigabitEthernet3
#  shutdown
# !
# interface GigabitEthernet4
#  shutdown
# !

- name: Parse configuration from device running config
  cisco.ios.ios_vrf_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task Output:
# ------------
#
# parsed:
#   - name: "GigabitEthernet1"
#     vrf_name: "vrf_C"
#   - name: "GigabitEthernet2"
#     vrf_name: "vrf_D"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"

# Using replaced

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  vrf forwarding vrf_A
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  vrf forwarding vrf_B
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  vrf forwarding vrf_C
#  no ip address
#  shutdown
#  negotiation auto

- name: Replace device configuration of listed VRF interfaces with provided configuration
  cisco.ios.ios_vrf_interfaces:
    config:
      - name: GigabitEthernet1
        vrf_name: vrf_D
      - name: GigabitEthernet2
        vrf_name: vrf_E
    state: replaced

# Task Output:
# ------------
#
# before:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#     vrf_name: "vrf_A"
#   - name: "GigabitEthernet2"
#     vrf_name: "vrf_B"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"
#     vrf_name: "vrf_C"
#
# commands:
#   - interface GigabitEthernet1
#   - no vrf forwarding vrf_A
#   - vrf forwarding vrf_D
#   - interface GigabitEthernet2
#   - no vrf forwarding vrf_B
#   - vrf forwarding vrf_E
#
# after:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#     vrf_name: "vrf_D"
#   - name: "GigabitEthernet2"
#     vrf_name: "vrf_E"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"
#     vrf_name: "vrf_C"

# Using deleted

# Before state:
# -------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  vrf forwarding vrf_A
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  vrf forwarding vrf_B
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  vrf forwarding vrf_C
#  no ip address
#  shutdown
#  negotiation auto

- name: Delete VRF configuration of specified interfaces
  cisco.ios.ios_vrf_interfaces:
    config:
      - name: GigabitEthernet1
      - name: GigabitEthernet2
    state: deleted

# Task Output:
# ------------
#
# before:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#     vrf_name: "vrf_A"
#   - name: "GigabitEthernet2"
#     vrf_name: "vrf_B"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"
#     vrf_name: "vrf_C"
#
# commands:
#   - interface GigabitEthernet1
#   - no vrf forwarding vrf_A
#   - interface GigabitEthernet2
#   - no vrf forwarding vrf_B
#
# after:
#   - name: "Loopback24"
#   - name: "GigabitEthernet1"
#   - name: "GigabitEthernet2"
#   - name: "GigabitEthernet3"
#   - name: "GigabitEthernet4"
#     vrf_name: "vrf_C"

# After state:
# ------------
#
# vios#show running-config | section ^interface
# interface Loopback24
#  no ip address
# interface GigabitEthernet1
#  ip address dhcp
#  negotiation auto
# interface GigabitEthernet2
#  no ip address
#  shutdown
#  negotiation auto
# interface GigabitEthernet3
#  no ip address
#  negotiation auto
# interface GigabitEthernet4
#  vrf forwarding vrf_C
#  no ip address
#  shutdown
#  negotiation auto
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted)
  type: list
  sample: >
    [
        {
            "name": "Loopback24"
        },
        {
            "name": "GigabitEthernet1"
        },
        {
            "name": "GigabitEthernet2",
            "vrf_name": "vrf_B"
        },
        {
            "name": "GigabitEthernet3"
        },
        {
            "name": "GigabitEthernet4"
        }
    ]

after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: list
  sample: >
    [
        {
            "name": "Loopback24"
        },
        {
            "name": "GigabitEthernet1"
        },
        {
            "name": "GigabitEthernet2",
            "vrf_name": "vrf_D"
        },
        {
            "name": "GigabitEthernet3"
        },
        {
            "name": "GigabitEthernet4"
        }
    ]

commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted)
  type: list
  sample:
    - "interface GigabitEthernet2"
    - "vrf forwarding vrf_D"
    - "no vrf forwarding vrf_B"

rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - "interface GigabitEthernet1"
    - "vrf forwarding vrf_C"
    - "interface GigabitEthernet2"
    - "vrf forwarding vrf_D"

gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    [
        {
            "name": "Loopback24"
        },
        {
            "name": "GigabitEthernet1"
        },
        {
            "name": "GigabitEthernet2",
            "vrf_name": "vrf_B"
        },
        {
            "name": "GigabitEthernet3"
        },
        {
            "name": "GigabitEthernet4"
        }
    ]

parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    [
        {
            "name": "GigabitEthernet1",
            "vrf_name": "vrf_C"
        },
        {
            "name": "GigabitEthernet2",
            "vrf_name": "vrf_D"
        },
        {
            "name": "GigabitEthernet3"
        },
        {
            "name": "GigabitEthernet4"
        }
    ]
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.vrf_interfaces.vrf_interfaces import (
    Vrf_interfacesArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.vrf_interfaces.vrf_interfaces import (
    Vrf_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Vrf_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Vrf_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2024, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: parse_serial
short_description: Convert a serial number as a colon-separated list of hex numbers to an integer
author: Felix Fontein (@felixfontein)
version_added: 2.18.0
description:
  - Parses a colon-separated list of hex numbers of the form C(00:11:22:33) and returns the corresponding integer.
options:
  _input:
    description:
      - A serial number represented as a colon-separated list of hex numbers between 0 and 255.
      - These numbers are interpreted as the byte presentation of an unsigned integer in network byte order. That is, C(01:00)
        is interpreted as the integer 256.
    type: string
    required: true
seealso:
  - plugin: community.crypto.to_serial
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Parse serial number
  ansible.builtin.debug:
    msg: "{{ '11:22:33' | community.crypto.parse_serial }}"
"""

RETURN = r"""
_value:
  description:
    - The serial number as an integer.
  type: int
"""

from collections.abc import Callable

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.crypto.plugins.module_utils._serial import (
    parse_serial,
)


def parse_serial_filter(serial_str: str | bytes) -> int:
    if not isinstance(serial_str, (str, bytes)):
        raise AnsibleFilterError(
            f"The input for the community.crypto.parse_serial filter must be a string; got {type(serial_str)} instead"
        )
    try:
        return parse_serial(to_text(serial_str))
    except ValueError as exc:
        raise AnsibleFilterError(str(exc)) from exc


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "parse_serial": parse_serial_filter,
        }



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Prefix_lists parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


# diable no-self-use
# pylint: disable=R0201
# pylint: disable=W0642
# pylint: disable=no-self-argument


class Prefix_listsTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Prefix_listsTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    def _tmplt_prefix_list_ip(config_data):
        command_set = []
        config_data = config_data["prefix_lists"].get("entries", {})
        for k, v in config_data.items():
            command = ""
            if k != "seq":
                command = "seq " + str(k) + " {action} {address}".format(**v)
            else:
                command = "{action} {address}".format(**v)
            if "match" in v:
                command += " {operator} {masklen}".format(**v["match"])
            if command:
                command_set.append(command)

        return command_set

    def _tmplt_prefix_list_ip_del(config_data):
        command_set = []
        config_data = config_data["prefix_lists"].get("entries", {})
        for k, v in config_data.items():
            command_set.append("seq " + str(k))

        return command_set

    def _tmplt_prefix_list_resequence(config_data):
        command = "resequence"
        config_data = config_data["prefix_lists"].get("entries", {})
        for k, v in config_data.items():
            if v["resequence"].get("start_seq"):
                command += " " + str(v["resequence"]["start_seq"])
            if v["resequence"].get("step"):
                command += " " + str(v["resequence"]["step"])

        return command

    # fmt: off
    PARSERS = [
        {
            "name": "prefixlist.name",
            "getval": re.compile(
                r"""
                ^(?P<afi>ip|ipv6)\sprefix-list\s(?P<name>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": '{{ "ip" if afi == "ipv4" else afi }} prefix-list {{ prefix_lists.name }}',
            "compval": "prefix_lists",
            "result": {
                '{{ afi  }}': {
                    "afi": '{{ "ipv4" if afi == "ip" else afi }}',
                    "prefix_lists": {
                        "{{ name }}": {
                            "name": "{{ name }}",
                        },
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "prefixlist.entry",
            "getval": re.compile(
                r"""
                \s*seq
                \s(?P<num>\d+)
                \s+(?P<action>permit|deny)
                \s+(?P<ip>\S+)
                \s*(?P<oper>eq|ge|le)*
                \s*(?P<len>\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_prefix_list_ip,
            "remval": _tmplt_prefix_list_ip_del,
            "compval": "prefix_lists",
            "result": {
                "{{ afi }}": {
                    "prefix_lists": {
                        "{{ name }}": {
                            "entries": {
                                '{{ num|d("seq") }}': {
                                    "sequence": "{{ num }}",
                                    "action": "{{ action }}",
                                    "address": "{{ ip }}",
                                    "match": {
                                        "operator": "{{ oper }}",
                                        "masklen": "{{ len }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "prefixlist.resequence",
            "getval": re.compile(
                r"""
                \s*resequence
                \s*(?P<start>\d+)*
                \*(?P<step>\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_prefix_list_resequence,
            "compval": "prefix_lists",
            "result": {
                "{{ afi }}": {
                    "prefix_lists": {
                        "{{ name }}": {
                            "entries": {
                                '{{ num|d("seq") }}': {
                                    "resequence": {
                                        "default": "{{ True if start_seq is undefined and step is undefined }}",
                                        "start_seq": "{{ start }}",
                                        "step": "{{ step }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2026, Rafal Kozlowski (@rkozlo) <rafalkozlowski07@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r'''
---
module: clickhouse_script

short_description: Run SQL queries from a file

description:
    - This module allows to execute SQL queries from files.
    - Queries has to be separated with C(;)
    - Module uses sqlglot for parsing file.
    - Module uses server side parametrization. Ex {d:Date}, {id:String}.

version_added: '2.2.0'

author:
  - Rafal Kozlowski (@rkozlo)

requirements:
  - sqlglot
  - clickhouse-driver

extends_documentation_fragment:
  - community.clickhouse.client_inst_opts

options:
  path:
    description:
      - Path to a SQL file to execute.
    type: str
    required: true
  parameters:
    description:
      - Parameters to pass to substitute in script.
      - Note module uses server side parametrization.
    type: dict

attributes:
  check_mode:
    description:
      - In check mode I(executed_statements) and I(query_parameters) will be set, but none queire will be sent to database.
    support: full
  idempotency:
    description: Always changed=True.
    support: none
'''

EXAMPLES = r'''
- name: Execute plain sql script
  community.clickhouse.clickhouse_script:
    path: test_file.sql

- name: Execute sql script with parametrization
  community.clickhouse.clickhouse_script:
    path: test_file.sql
    parameters:
      d: "2026-05-19"
      c: "cl"
'''

RETURN = r'''
executed_statements:
  description:
    - Queries executed from file. In case of failure, queries executed prior to the failed one are returned via RV(custom_message).
  returned: on success
  type: list
  sample: [
    "CREATE DATABASE IF NOT EXISTS test_db",
    "SELECT 1 FROM test_db WHERE b = {a:String}"
  ]
query_parameters:
  description:
    - Query parameters passed to server.
  returned: on success
  type: list
  sample: [
    {
      "a": "cl",
      "d": "2026-05-19"
    }
  ]
custom_message:
  description:
    - Queries that was successfully executed in case if one of them fail. Normally, executed queries are returned via RV(executed_statements).
  returned: on failure
  type: dict
  sample:
    success_queries:
      - SELECT 1
      - SELECT 1
'''

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_native
from pathlib import Path
try:
    import sqlglot
    HAS_SQLGLOT = True
    SQLGLOT_IMPORT_ERROR = None
except ImportError:
    HAS_SQLGLOT = False

from ansible_collections.community.clickhouse.plugins.module_utils.clickhouse import (
    get_main_conn_kwargs,
    connect_to_db_via_client,
    check_clickhouse_driver,
    client_common_argument_spec,
    execute_query
)

executed_statements = []
query_parameters = []


class ClickHouseScript():
    def __init__(self, module, client, path):
        self.module = module
        self.client = client
        self.path = Path(path)
        self._exists = None
        self._is_file = None

    @property
    def exists(self):
        if self._exists is None:
            self._exists = self.path.exists()
        return self._exists

    @property
    def is_file(self):
        if self._is_file is None:
            self._is_file = self.path.is_file()
        return self._is_file

    def _validate(self):
        if not self.exists:
            self.module.fail_json(msg=f"Passed path does not exists: {str(self.path)}")
        elif not self.is_file:
            self.module.fail_json(msg=f"Passed path is not regular file: {str(self.path)}")

    def _get_statements(self):
        '''Method returns AST structure.'''
        try:
            content = self.path.read_text()
        except Exception as e:
            self.module.fail_json(msg="Failed reading file : %s" % to_native(e))
        try:
            return sqlglot.parse(content, dialect="clickhouse")
        except Exception as e:
            self.module.fail_json(msg="Failed parsing file: %s" % to_native(e))

    def execute(self, parameters=None):
        self._validate()

        statements = self._get_statements()
        execute_kwargs = None
        if parameters:
            execute_kwargs = {
                'params': parameters,
                # To keep consistency between sqlglot and server.
                # Without this server expects client side parametrization %(d)s% format.
                'settings': {'server_side_params': True},
            }
            query_parameters.append(parameters)
        for stmt in statements:
            sql = stmt.sql(dialect="clickhouse")
            if not self.module.check_mode:
                execute_query(self.module, self.client, sql, execute_kwargs=execute_kwargs, custom_message={'success_queries': executed_statements})
            executed_statements.append(sql)


def main():
    # Set up arguments.
    # If there are common arguments shared across several modules,
    # create the common_argument_spec() function under plugins/module_utils/*
    # and invoke here to return a dict with those arguments
    argument_spec = client_common_argument_spec()
    argument_spec.update(
        path=dict(type="str", required=True),
        parameters=dict(type="dict")
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    # Assign passed options to variables
    client_kwargs = module.params["client_kwargs"]
    # The reason why these arguments are separate from client_kwargs
    # is that we need to protect some sensitive data like passwords passed
    # to the module from logging (see the arguments above with no_log=True);
    # Such data must be passed as module arguments (not nested deep in values).
    main_conn_kwargs = get_main_conn_kwargs(module)
    path = module.params["path"]
    parameters = module.params["parameters"]

    # Will fail if no driver informing the user
    check_clickhouse_driver(module)

    if not HAS_SQLGLOT:
        module.fail_json(msg=missing_required_lib('sqlglot'))

    # Connect to DB
    client = connect_to_db_via_client(module, main_conn_kwargs, client_kwargs)

    # Do the job
    clickhouse_script = ClickHouseScript(module=module, client=client, path=path)

    clickhouse_script.execute(parameters)

    # Close connection
    client.disconnect_connection()

    # Users will get this in JSON output after execution
    module.exit_json(changed=True, executed_statements=executed_statements, query_parameters=query_parameters)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2014, Nate Coraor <nate@bx.psu.edu>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: capabilities
short_description: Manage Linux capabilities
description:
  - This module manipulates files privileges using the Linux capabilities(7) system.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  path:
    description:
      - Specifies the path to the file to be managed.
    type: str
    required: true
    aliases: [key]
  capability:
    description:
      - Desired capability to set (with operator and flags, if O(state=present)) or remove (if O(state=absent)).
    type: str
    required: true
    aliases: [cap]
  state:
    description:
      - Whether the entry should be present or absent in the file's capabilities.
    type: str
    choices: [absent, present]
    default: present
notes:
  - The capabilities system automatically transforms operators and flags into the effective set, so for example, C(cap_foo=ep)
    probably becomes C(cap_foo+ep).
  - This module does not attempt to determine the final operator and flags to compare, so you want to ensure that your capabilities
    argument matches the final capabilities.
author:
  - Nate Coraor (@natefoo)
"""

EXAMPLES = r"""
- name: Set cap_sys_chroot+ep on /foo
  community.general.capabilities:
    path: /foo
    capability: cap_sys_chroot+ep
    state: present

- name: Remove cap_net_bind_service from /bar
  community.general.capabilities:
    path: /bar
    capability: cap_net_bind_service
    state: absent
"""

from ansible.module_utils.basic import AnsibleModule

OPS = ("=", "-", "+")


class CapabilitiesModule:
    platform = "Linux"
    distribution = None

    def __init__(self, module):
        self.module = module
        self.path = module.params["path"].strip()
        self.capability = module.params["capability"].strip().lower()
        self.state = module.params["state"]
        self.getcap_cmd = module.get_bin_path("getcap", required=True)
        self.setcap_cmd = module.get_bin_path("setcap", required=True)
        self.capability_tup = self._parse_cap(self.capability, op_required=self.state == "present")

        self.run()

    def run(self):
        current = self.getcap(self.path)
        caps = [cap[0] for cap in current]

        if self.state == "present" and self.capability_tup not in current:
            # need to add capability
            if self.module.check_mode:
                self.module.exit_json(changed=True, msg="capabilities changed")
            else:
                # remove from current cap list if it is already set (but op/flags differ)
                current = [x for x in current if x[0] != self.capability_tup[0]]
                # add new cap with correct op/flags
                current.append(self.capability_tup)
                self.module.exit_json(
                    changed=True, state=self.state, msg="capabilities changed", stdout=self.setcap(self.path, current)
                )
        elif self.state == "absent" and self.capability_tup[0] in caps:
            # need to remove capability
            if self.module.check_mode:
                self.module.exit_json(changed=True, msg="capabilities changed")
            else:
                # remove from current cap list and then set current list
                current = [x for x in current if x[0] != self.capability_tup[0]]
                self.module.exit_json(
                    changed=True, state=self.state, msg="capabilities changed", stdout=self.setcap(self.path, current)
                )
        self.module.exit_json(changed=False, state=self.state)

    def getcap(self, path):
        rval = []
        cmd = [self.getcap_cmd, "-v", path]
        rc, stdout, stderr = self.module.run_command(cmd)
        # If file xattrs are set but no caps are set the output will be:
        #   '/foo ='
        # If file xattrs are unset the output will be:
        #   '/foo'
        # If the file does not exist, the stderr will be (with rc == 0...):
        #   '/foo (No such file or directory)'
        if rc != 0 or stderr != "":
            self.module.fail_json(msg=f"Unable to get capabilities of {path}", stdout=stdout.strip(), stderr=stderr)
        if stdout.strip() != path:
            if " =" in stdout:
                # process output of an older version of libcap
                caps = stdout.split(" =")[1].strip().split()
            elif stdout.strip().endswith(")"):  # '/foo (Error Message)'
                self.module.fail_json(msg=f"Unable to get capabilities of {path}", stdout=stdout.strip(), stderr=stderr)
            else:
                # otherwise, we have a newer version here
                # see original commit message of cap/v0.2.40-18-g177cd41 in libcap.git
                caps = stdout.split()[1].strip().split()
            for cap in caps:
                cap = cap.lower()
                # getcap condenses capabilities with the same op/flags into a
                # comma-separated list, so we have to parse that
                if "," in cap:
                    cap_group = cap.split(",")
                    cap_group[-1], op, flags = self._parse_cap(cap_group[-1])
                    for subcap in cap_group:
                        rval.append((subcap, op, flags))
                else:
                    rval.append(self._parse_cap(cap))
        return rval

    def setcap(self, path, caps):
        caps = " ".join(["".join(cap) for cap in caps])
        cmd = [self.setcap_cmd, caps, path]
        rc, stdout, stderr = self.module.run_command(cmd)
        if rc != 0:
            self.module.fail_json(msg=f"Unable to set capabilities of {path}", stdout=stdout, stderr=stderr)
        else:
            return stdout

    def _parse_cap(self, cap, op_required=True):
        opind = -1
        try:
            i = 0
            while opind == -1:
                opind = cap.find(OPS[i])
                i += 1
        except Exception:
            if op_required:
                self.module.fail_json(msg=f"Couldn't find operator (one of: {OPS})")
            else:
                return (cap, None, None)
        op = cap[opind]
        cap, flags = cap.split(op)
        return (cap, op, flags)


# ==============================================================
# main


def main():
    # defining module
    module = AnsibleModule(
        argument_spec=dict(
            path=dict(type="str", required=True, aliases=["key"]),
            capability=dict(type="str", required=True, aliases=["cap"]),
            state=dict(type="str", default="present", choices=["absent", "present"]),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    CapabilitiesModule(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: hosttech_dns_record_info

short_description: Retrieve records in Hosttech DNS service

version_added: 2.0.0

description:
  - Retrieves DNS records in Hosttech DNS service.
extends_documentation_fragment:
  - community.dns._hosttech
  - community.dns._hosttech.record_type_choices
  - community.dns._hosttech.record_type_seealso
  - community.dns._hosttech.zone_id_type
  - community.dns._module_record_info
  - community.dns._options.record_transformation
  - community.dns._attributes
  - community.dns._attributes.actiongroup_hosttech
  - community.dns._attributes.info_module
  - community.dns._attributes.idempotent_not_modify_state

attributes:
  action_group:
    version_added: 2.4.0

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.dns.hosttech_dns_record_set_info
  - plugin: community.dns.hosttech_dns_records
    plugin_type: inventory
"""

EXAMPLES = r"""
- name: Retrieve the details for the A records of new.foo.com
  community.dns.hosttech_dns_record_info:
    zone_name: foo.com
    record: new.foo.com
    type: A
    hosttech_token: access_token
  register: rec

- name: Print the A records
  ansible.builtin.debug:
    msg: "{{ rec.records }}"
"""

RETURN = r"""
records:
  description: The list of fetched records.
  type: list
  elements: dict
  returned: success and O(what) is not V(single_record)
  contains:
    record:
      description: The record name.
      type: str
      sample: sample.example.com
    prefix:
      description: The record prefix.
      type: str
      sample: sample
    type:
      description: The DNS record type.
      type: str
      sample: A
    ttl:
      description:
        - The TTL.
      type: int
      sample: 3600
    value:
      description: The DNS record's value.
      type: str
      sample: 1.2.3.4
    extra:
      description: Extra information on records.
      type: dict
      sample:
        comment: ''
  sample:
    - record: sample.example.com
      type: A
      ttl: 3600
      value: 1.2.3.4
      extra: {}

zone_id:
  description: The ID of the zone.
  type: int
  returned: success
  sample: 23
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.dns.plugins.module_utils._argspec import (
    ModuleOptionProvider,
)
from ansible_collections.community.dns.plugins.module_utils._hosttech.api import (
    create_hosttech_api,
    create_hosttech_argument_spec,
    create_hosttech_provider_information,
)
from ansible_collections.community.dns.plugins.module_utils._http import (
    ModuleHTTPHelper,
)
from ansible_collections.community.dns.plugins.module_utils._module.record_info import (
    create_module_argument_spec,
    run_module,
)


def main() -> None:
    provider_information = create_hosttech_provider_information()
    argument_spec = create_hosttech_argument_spec()
    argument_spec.merge(
        create_module_argument_spec(provider_information=provider_information)
    )
    module = AnsibleModule(supports_check_mode=True, **argument_spec.to_kwargs())
    run_module(
        module,
        lambda: create_hosttech_api(
            ModuleOptionProvider(module), ModuleHTTPHelper(module)
        ),
        provider_information=provider_information,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Jose Angel Munoz, <josea.munoz@gmail.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_keyvaultsecret_info
version_added: "0.1.2"
short_description: Get Azure Key Vault secret facts
description:
    - Get facts of Azure Key Vault secret.

options:
    vault_uri:
        description:
            - Vault uri where the secret stored in.
        required: True
        type: str
    name:
        description:
            - Secret name. If not set, will list all enabled secrets in vault_uri (disabled secrets will be skipped).
        type: str
    version:
        description:
            - Secret version.
            - Set it to C(current) to show latest version of a secret.
            - Set it to C(all) to list all versions of a secret.
            - Set it to specific version to list specific version of a secret. eg. fd2682392a504455b79c90dd04a1bf46
        default: current
        type: str
    show_deleted_secret:
        description:
            - Set to I(show_delete_secret=true) to show deleted secrets. Set to I(show_deleted_secret=false) to show not deleted secrets.
        type: bool
        default: false
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jose Angel Munoz (@imjoseangel)

'''

EXAMPLES = '''
- name: Get latest version of specific secret
  azure_rm_keyvaultsecret_info:
    vault_uri: "https://myVault.vault.azure.net"
    name: mySecret

- name: List all versions of specific secret
  azure_rm_keyvaultsecret_info:
    vault_uri: "https://myVault.vault.azure.net"
    name: mySecret
    version: all

- name: List specific version of specific secret
  azure_rm_keyvaultsecret_info:
    vault_uri: "https://myVault.vault.azure.net"
    name: mySecret
    version: fd2682392a504455b79c90dd04a1bf46

- name: List all secrets in specific key vault
  azure_rm_keyvaultsecret_info:
    vault_uri: "https://myVault.vault.azure.net"

- name: List deleted secrets in specific key vault
  azure_rm_keyvaultsecret_info:
    vault_uri: "https://myVault.vault.azure.net"
    show_deleted_secret: true
'''

RETURN = '''
secrets:
    description:
        - List of secrets in Azure Key Vault.
    returned: always
    type: complex
    contains:
        sid:
            description:
                - Secret identifier.
            returned: always
            type: str
            sample: "https://myVault.vault.azure.net/flexsecret/secret1/fd2682392a504455b79c90dd04a1bf46"
        version:
            description:
                - Secret version.
            type: str
            returned: always
            sample: fd2682392a504455b79c90dd04a1bf46
        secret:
            description: secret value.
            type: str
            returned: always
            sample: mysecretvault
        tags:
            description:
                - Tags of the secret.
            returned: always
            type: dict
            sample: {"delete": "on-exit"}
        content_type:
            description:
                - Content type (optional)
            returned: always
            type: str
            sample: mysecrettype
        attributes:
            description:
                - Secret attributes.
            type: dict
            contains:
                created:
                    description:
                        - Creation datetime.
                    returned: always
                    type: str
                    sample: "2019-04-25T07:26:49+00:00"
                not_before:
                    description:
                        - Not before datetime.
                    type: str
                    sample: "2019-04-25T07:26:49+00:00"
                expires:
                    description:
                        - Expiration datetime.
                    type: str
                    sample: "2019-04-25T07:26:49+00:00"
                updated:
                    description:
                        - Update datetime.
                    returned: always
                    type: str
                    sample: "2019-04-25T07:26:49+00:00"
                enabled:
                    description:
                        - Indicate whether the secret is enabled.
                    returned: always
                    type: str
                    sample: true
                recovery_level:
                    description:
                        - Reflects the deletion recovery level currently in effect for secrets in the current vault.
                        - If it contains 'Purgeable' the secret can be permanently deleted by a privileged user,
                        - Otherwise, only the system can purge the secret, at the end of the retention interval.
                    returned: always
                    type: str
                    sample: Recoverable+Purgeable
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.keyvault.secrets import SecretClient
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


def secretbundle_to_dict(bundle):

    return dict(tags=bundle._properties._tags,
                attributes=dict(
                    enabled=bundle._properties._attributes.enabled,
                    not_before=bundle._properties._attributes.not_before,
                    expires=bundle._properties._attributes.expires,
                    created=bundle._properties._attributes.created,
                    updated=bundle._properties._attributes.updated,
                    recovery_level=bundle._properties._attributes.recovery_level),
                sid=bundle._properties._id,
                version=bundle._properties.version,
                content_type=bundle._properties._content_type,
                secret=bundle._value)


def deleted_bundle_to_dict(bundle):
    return dict(tags=bundle.tags,
                attributes=dict(
                    enabled=bundle.enabled,
                    not_before=bundle.not_before,
                    expires=bundle.expires_on,
                    created=bundle.created_on,
                    updated=bundle.updated_on,
                    recovery_level=bundle.recovery_level),
                sid=bundle.id,
                version=bundle.key_id,
                content_type=bundle.content_type,
                secret=bundle.version)


def deletedsecretbundle_to_dict(bundle):
    secretbundle = deleted_bundle_to_dict(bundle.properties)
    secretbundle['recovery_id'] = bundle._recovery_id
    secretbundle['scheduled_purge_date'] = bundle._scheduled_purge_date
    secretbundle['deleted_date'] = bundle._deleted_date
    return secretbundle


def secretitem_to_dict(secretitem):
    return dict(sid=secretitem._id,
                version=secretitem.version,
                tags=secretitem._tags,
                attributes=dict(
                    enabled=secretitem._attributes.enabled,
                    not_before=secretitem._attributes.not_before,
                    expires=secretitem._attributes.expires,
                    created=secretitem._attributes.created,
                    updated=secretitem._attributes.updated,
                    recovery_level=secretitem._attributes.recovery_level))


def deletedsecretitem_to_dict(secretitem):
    item = secretitem_to_dict(secretitem.properties)
    item['recovery_id'] = secretitem._recovery_id
    item['scheduled_purge_date'] = secretitem._scheduled_purge_date
    item['deleted_date'] = secretitem._deleted_date
    return item


class AzureRMKeyVaultSecretInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(version=dict(type='str',
                                                 default='current'),
                                    name=dict(type='str'),
                                    vault_uri=dict(type='str', required=True),
                                    show_deleted_secret=dict(type='bool',
                                                             default=False),
                                    tags=dict(type='list', elements='str'))

        self.vault_uri = None
        self.name = None
        self.version = None
        self.show_deleted_secret = False
        self.tags = None

        self.results = dict(changed=False)
        self._client = None

        super(AzureRMKeyVaultSecretInfo,
              self).__init__(derived_arg_spec=self.module_arg_spec,
                             supports_check_mode=True,
                             supports_tags=False,
                             facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for secret in list(self.module_arg_spec.keys()):
            if hasattr(self, secret):
                setattr(self, secret, kwargs[secret])

        self._client = self.get_keyvault_client()

        if self.name:
            if self.show_deleted_secret:
                self.results['secrets'] = self.get_deleted_secret()
            else:
                if self.version == 'all':
                    self.results['secrets'] = self.get_secret_versions()
                else:
                    response = self.get_secret(self.name)
                    if response is not None:
                        self.results['secrets'] = [response]
                    else:
                        self.results['secrets'] = []
        else:
            if self.show_deleted_secret:
                self.results['secrets'] = self.list_deleted_secrets()
            else:
                self.results['secrets'] = self.list_secrets()

        return self.results

    def get_keyvault_client(self):

        return SecretClient(vault_url=self.vault_uri, credential=self.azure_auth.azure_credential_track2)

    def get_secret(self, name):
        '''
        Gets the properties of the specified secret in key vault.

        :return: deserialized secret state dictionary
        '''
        self.log("Get the secret {0}".format(name))

        results = None
        try:
            if self.version == 'current':
                response = self._client.get_secret(name=name, version='')
            else:
                response = self._client.get_secret(name=name, version=self.version)

            if response:
                response = secretbundle_to_dict(response)
                if self.has_tags(response['tags'], self.tags):
                    self.log("Response : {0}".format(response))
                    results = response

        except ResourceNotFoundError as ec:
            self.log("Did not find the key vault secret {0}: {1}".format(
                name, str(ec)))
        except Exception as ec2:
            self.fail("Find the key vault secret got exception, exception as {0}".format(str(ec2)))
        return results

    def get_secret_versions(self):
        '''
        Lists secrets versions.

        :return: deserialized versions of secret, includes secret identifier, attributes and tags
        '''
        self.log("Get the secret versions {0}".format(self.name))

        results = []
        try:
            response = self._client.list_properties_of_secret_versions(name=self.name)
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    item = secretitem_to_dict(item)
                    if self.has_tags(item['tags'], self.tags):
                        results.append(item)
        except Exception as e:
            self.fail("Did not find secret versions {0} : {1}.".format(
                self.name, str(e)))
        return results

    def list_secrets(self):
        '''
        Lists secrets in specific key vault.

        :return: deserialized secrets, includes secret identifier, attributes and tags.
        '''
        self.log("Get the key vaults in current subscription")

        results = []
        try:
            response = self._client.list_properties_of_secrets()
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    # The API will raise exception if the secret is disabled.
                    # Exception as (Forbidden) Operation get is not allowed on a disabled secret.
                    item = self.get_secret(item._id.split('/')[-1]) if item._attributes.enabled else None
                    if item is not None:
                        results.append(item)
        except Exception as e:
            self.fail(
                "Did not find key vault in current subscription {0}.".format(
                    str(e)))
        return results

    def get_deleted_secret(self):
        '''
        Gets the properties of the specified deleted secret in key vault.

        :return: deserialized secret state dictionary
        '''
        self.log("Get the secret {0}".format(self.name))

        results = []
        try:
            response = self._client.get_deleted_secret(name=self.name)

            if response:
                response = deletedsecretbundle_to_dict(response)
                if self.has_tags(response['tags'], self.tags):
                    self.log("Response : {0}".format(response))
                    results.append(response)

        except ResourceNotFoundError as ec:
            self.log("Did not find the key vault secret {0}: {1}".format(
                self.name, str(ec)))
        except Exception as ec2:
            self.fail("Did not find the key vault secret {0}: {1}".format(
                self.name, str(ec2)))
        return results

    def list_deleted_secrets(self):
        '''
        Lists deleted secrets in specific key vault.

        :return: deserialized secrets, includes secret identifier, attributes and tags.
        '''
        self.log("Get the key vaults in current subscription")

        results = []
        try:
            response = self._client.list_deleted_secrets()
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    item = deletedsecretitem_to_dict(item)
                    if self.has_tags(item['tags'], self.tags):
                        results.append(item)
        except Exception as e:
            self.fail(
                "Did not find key vault in current subscription {0}.".format(
                    str(e)))
        return results


def main():
    """Main execution"""
    AzureRMKeyVaultSecretInfo()


if __name__ == '__main__':
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import base64
import collections
import json
import os
import os.path
import shlex
import string
import typing as t
from urllib.parse import urlparse, urlunparse

from ansible_collections.community.docker.plugins.module_utils._version import (
    StrictVersion,
)

from .. import errors
from ..constants import (
    BYTE_UNITS,
    DEFAULT_HTTP_HOST,
    DEFAULT_NPIPE,
    DEFAULT_UNIX_SOCKET,
)
from ..tls import TLSConfig

if t.TYPE_CHECKING:
    from collections.abc import Mapping, Sequence


URLComponents = collections.namedtuple(
    "URLComponents",
    "scheme netloc url params query fragment",
)


def decode_json_header(header: str | bytes) -> dict[str, t.Any]:
    data = base64.b64decode(header).decode("utf-8")
    return json.loads(data)


def compare_version(v1: str, v2: str) -> t.Literal[-1, 0, 1]:
    """Compare docker versions

    >>> v1 = '1.9'
    >>> v2 = '1.10'
    >>> compare_version(v1, v2)
    1
    >>> compare_version(v2, v1)
    -1
    >>> compare_version(v2, v2)
    0
    """
    s1 = StrictVersion(v1)
    s2 = StrictVersion(v2)
    if s1 == s2:
        return 0
    if s1 > s2:
        return -1
    return 1


def version_lt(v1: str, v2: str) -> bool:
    return compare_version(v1, v2) > 0


def version_gte(v1: str, v2: str) -> bool:
    return not version_lt(v1, v2)


def _convert_port_binding(
    binding: (
        tuple[str, str | int | None]
        | tuple[str | int | None]
        | dict[str, str]
        | str
        | int
    ),
) -> dict[str, str]:
    result = {"HostIp": "", "HostPort": ""}
    host_port: str | int | None = ""
    if isinstance(binding, tuple):
        if len(binding) == 2:
            host_port = binding[1]  # type: ignore
            result["HostIp"] = binding[0]
        elif isinstance(binding[0], str):
            result["HostIp"] = binding[0]
        else:
            host_port = binding[0]
    elif isinstance(binding, dict):
        if "HostPort" in binding:
            host_port = binding["HostPort"]
            if "HostIp" in binding:
                result["HostIp"] = binding["HostIp"]
        else:
            raise ValueError(binding)
    else:
        host_port = binding

    result["HostPort"] = str(host_port) if host_port is not None else ""
    return result


def convert_port_bindings(
    port_bindings: dict[
        str | int,
        tuple[str, str | int | None]
        | tuple[str | int | None]
        | dict[str, str]
        | str
        | int
        | list[
            tuple[str, str | int | None]
            | tuple[str | int | None]
            | dict[str, str]
            | str
            | int
        ],
    ],
) -> dict[str, list[dict[str, str]]]:
    result = {}
    for k, v in port_bindings.items():
        key = str(k)
        if "/" not in key:
            key += "/tcp"
        if isinstance(v, list):
            result[key] = [_convert_port_binding(binding) for binding in v]
        else:
            result[key] = [_convert_port_binding(v)]
    return result


def convert_volume_binds(
    binds: (
        list[str]
        | Mapping[
            str | bytes, dict[str, str | bytes] | dict[str, str] | bytes | str | int
        ]
    ),
) -> list[str]:
    if isinstance(binds, list):
        return binds  # type: ignore

    result = []
    for k, v in binds.items():
        if isinstance(k, bytes):
            k = k.decode("utf-8")

        if isinstance(v, dict):
            if "ro" in v and "mode" in v:
                raise ValueError(f'Binding cannot contain both "ro" and "mode": {v!r}')

            bind = v["bind"]
            if isinstance(bind, bytes):
                bind = bind.decode("utf-8")

            if "ro" in v:
                mode = "ro" if v["ro"] else "rw"
            elif "mode" in v:
                mode = v["mode"]  # type: ignore # TODO
            else:
                mode = "rw"

            # NOTE: this is only relevant for Linux hosts
            # (does not apply in Docker Desktop)
            propagation_modes = [
                "rshared",
                "shared",
                "rslave",
                "slave",
                "rprivate",
                "private",
            ]
            if "propagation" in v and v["propagation"] in propagation_modes:
                if mode:
                    mode = ",".join([mode, v["propagation"]])  # type: ignore # TODO
                else:
                    mode = v["propagation"]  # type: ignore # TODO

            result.append(f"{k}:{bind}:{mode}")
        else:
            if isinstance(v, bytes):
                v = v.decode("utf-8")
            result.append(f"{k}:{v}:rw")
    return result


def convert_tmpfs_mounts(tmpfs: dict[str, str] | list[str]) -> dict[str, str]:
    if isinstance(tmpfs, dict):
        return tmpfs

    if not isinstance(tmpfs, list):
        raise ValueError(
            f"Expected tmpfs value to be either a list or a dict, found: {type(tmpfs).__name__}"
        )

    result = {}
    for mount in tmpfs:
        if isinstance(mount, str):
            if ":" in mount:
                name, options = mount.split(":", 1)
            else:
                name = mount
                options = ""

        else:
            raise ValueError(
                f"Expected item in tmpfs list to be a string, found: {type(mount).__name__}"
            )

        result[name] = options
    return result


def convert_service_networks(
    networks: list[str | dict[str, str]],
) -> list[dict[str, str]]:
    if not networks:
        return networks  # type: ignore
    if not isinstance(networks, list):
        raise TypeError("networks parameter must be a list.")

    result = []
    for n in networks:
        if isinstance(n, str):
            n = {"Target": n}
        result.append(n)
    return result


def parse_repository_tag(repo_name: str) -> tuple[str, str | None]:
    parts = repo_name.rsplit("@", 1)
    if len(parts) == 2:
        return tuple(parts)  # type: ignore
    parts = repo_name.rsplit(":", 1)
    if len(parts) == 2 and "/" not in parts[1]:
        return tuple(parts)  # type: ignore
    return repo_name, None


def parse_host(addr: str | None, is_win32: bool = False, tls: bool = False) -> str:
    # Sensible defaults
    if not addr and is_win32:
        return DEFAULT_NPIPE
    if not addr or addr.strip() == "unix://":
        return DEFAULT_UNIX_SOCKET

    addr = addr.strip()

    parsed_url = urlparse(addr)
    proto = parsed_url.scheme
    if not proto or any(x not in string.ascii_letters + "+" for x in proto):
        # https://bugs.python.org/issue754016
        parsed_url = urlparse("//" + addr, "tcp")
        proto = "tcp"

    if proto == "fd":
        raise errors.DockerException("fd protocol is not implemented")

    # These protos are valid aliases for our library but not for the
    # official spec
    if proto in ("http", "https"):
        tls = proto == "https"
        proto = "tcp"
    elif proto == "http+unix":
        proto = "unix"

    if proto not in ("tcp", "unix", "npipe", "ssh"):
        raise errors.DockerException(f"Invalid bind address protocol: {addr}")

    if proto == "tcp" and not parsed_url.netloc:
        # "tcp://" is exceptionally disallowed by convention;
        # omitting a hostname for other protocols is fine
        raise errors.DockerException(f"Invalid bind address format: {addr}")

    if any(
        [parsed_url.params, parsed_url.query, parsed_url.fragment, parsed_url.password]
    ):
        raise errors.DockerException(f"Invalid bind address format: {addr}")

    if parsed_url.path and proto == "ssh":
        raise errors.DockerException(
            f"Invalid bind address format: no path allowed for this protocol: {addr}"
        )
    path = parsed_url.path
    if proto == "unix" and parsed_url.hostname is not None:
        # For legacy reasons, we consider unix://path
        # to be valid and equivalent to unix:///path
        path = f"{parsed_url.hostname}/{path}"

    netloc = parsed_url.netloc
    if proto in ("tcp", "ssh"):
        port = parsed_url.port or 0
        if port <= 0:
            port = 22 if proto == "ssh" else (2375 if tls else 2376)
            netloc = f"{parsed_url.netloc}:{port}"

        if not parsed_url.hostname:
            netloc = f"{DEFAULT_HTTP_HOST}:{port}"

    # Rewrite schemes to fit library internals (requests adapters)
    if proto == "tcp":
        proto = f"http{'s' if tls else ''}"
    elif proto == "unix":
        proto = "http+unix"

    if proto in ("http+unix", "npipe"):
        return f"{proto}://{path}".rstrip("/")
    return urlunparse(
        URLComponents(
            scheme=proto,
            netloc=netloc,
            url=path,
            params="",
            query="",
            fragment="",
        )
    ).rstrip("/")


def parse_devices(devices: Sequence[dict[str, str] | str]) -> list[dict[str, str]]:
    device_list = []
    for device in devices:
        if isinstance(device, dict):
            device_list.append(device)
            continue
        if not isinstance(device, str):
            raise errors.DockerException(f"Invalid device type {type(device)}")
        device_mapping = device.split(":")
        if device_mapping:
            path_on_host = device_mapping[0]
            if len(device_mapping) > 1:
                path_in_container = device_mapping[1]
            else:
                path_in_container = path_on_host
            if len(device_mapping) > 2:
                permissions = device_mapping[2]
            else:
                permissions = "rwm"
            device_list.append(
                {
                    "PathOnHost": path_on_host,
                    "PathInContainer": path_in_container,
                    "CgroupPermissions": permissions,
                }
            )
    return device_list


def kwargs_from_env(
    assert_hostname: bool | None = None,
    environment: Mapping[str, str] | None = None,
) -> dict[str, t.Any]:
    if not environment:
        environment = os.environ
    host = environment.get("DOCKER_HOST")

    # empty string for cert path is the same as unset.
    cert_path = environment.get("DOCKER_CERT_PATH") or None

    # empty string for tls verify counts as "false".
    # Any value or 'unset' counts as true.
    tls_verify_str = environment.get("DOCKER_TLS_VERIFY")
    if tls_verify_str == "":
        tls_verify = False
    else:
        tls_verify = tls_verify_str is not None
    enable_tls = cert_path or tls_verify

    params: dict[str, t.Any] = {}

    if host:
        params["base_url"] = host

    if not enable_tls:
        return params

    if not cert_path:
        cert_path = os.path.join(os.path.expanduser("~"), ".docker")

    if not tls_verify and assert_hostname is None:
        # assert_hostname is a subset of TLS verification,
        # so if it is not set already then set it to false.
        assert_hostname = False

    params["tls"] = TLSConfig(
        client_cert=(
            os.path.join(cert_path, "cert.pem"),
            os.path.join(cert_path, "key.pem"),
        ),
        ca_cert=os.path.join(cert_path, "ca.pem"),
        verify=tls_verify,
        assert_hostname=assert_hostname,
    )

    return params


def convert_filters(
    filters: Mapping[str, bool | str | int | list[int] | list[str] | list[str | int]],
) -> str:
    result = {}
    for k, v in filters.items():
        if isinstance(v, bool):
            v = "true" if v else "false"
        if not isinstance(v, list):
            v = [
                v,
            ]
        result[k] = [str(item) if not isinstance(item, str) else item for item in v]
    return json.dumps(result)


def parse_bytes(s: int | float | str) -> int | float:
    if isinstance(s, (int, float)):
        return s
    if len(s) == 0:
        return 0

    if s[-2:-1].isalpha() and s[-1].isalpha() and (s[-1] == "b" or s[-1] == "B"):
        s = s[:-1]
    units = BYTE_UNITS
    suffix = s[-1].lower()

    # Check if the variable is a string representation of an int
    # without a units part. Assuming that the units are bytes.
    if suffix.isdigit():
        digits_part = s
        suffix = "b"
    else:
        digits_part = s[:-1]

    if suffix in units or suffix.isdigit():
        try:
            digits = float(digits_part)
        except ValueError as exc:
            raise errors.DockerException(
                f"Failed converting the string value for memory ({digits_part}) to an integer."
            ) from exc

        # Reconvert to long for the final result
        s = int(digits * units[suffix])
    else:
        raise errors.DockerException(
            f"The specified value for memory ({s}) should specify the units. The postfix should be one of the `b` `k` `m` `g` characters"
        )

    return s


def normalize_links(links: dict[str, str] | Sequence[tuple[str, str]]) -> list[str]:
    if isinstance(links, dict):
        sorted_links = sorted(links.items())
    else:
        sorted_links = sorted(links)

    return [f"{k}:{v}" if v else k for k, v in sorted_links]


def parse_env_file(env_file: str | os.PathLike) -> dict[str, str]:
    """
    Reads a line-separated environment file.
    The format of each line should be "key=value".
    """
    environment = {}

    with open(env_file, "rt", encoding="utf-8") as f:
        for line in f:
            if line[0] == "#":
                continue

            line = line.strip()
            if not line:
                continue

            parse_line = line.split("=", 1)
            if len(parse_line) == 2:
                k, v = parse_line
                environment[k] = v
            else:
                raise errors.DockerException(
                    f"Invalid line in environment file {env_file}:\n{line}"
                )

    return environment


def split_command(command: str) -> list[str]:
    return shlex.split(command)


def format_environment(environment: Mapping[str, str | bytes | None]) -> list[str]:
    def format_env(key: str, value: str | bytes | None) -> str:
        if value is None:
            return key
        if isinstance(value, bytes):
            value = value.decode("utf-8")

        return f"{key}={value}"

    return [format_env(*var) for var in environment.items()]


def format_extra_hosts(extra_hosts: Mapping[str, str], task: bool = False) -> list[str]:
    # Use format dictated by Swarm API if container is part of a task
    if task:
        return [f"{v} {k}" for k, v in sorted(extra_hosts.items())]

    return [f"{k}:{v}" for k, v in sorted(extra_hosts.items())]



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_batch_deployment_info
version_added: "3.18.0"
short_description: List ML Batch Deployment resources
description:
    - List ML Batch Deployment resources.
options:
    name:
        description:
            - Name of the deployment.
        type: str
        required: false
    endpoint_name:
        description:
            - Name of the endpoint.
        required: true
        type: str
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get Specific ML Batch Deployment by name
  azure.azcollection.azure_rm_ml_batch_deployment_info:
    name: deployment-command-xxxxxxxxxx
    endpoint_name: endpoint-command-xxxxxxxxxx
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List ML Batch Deployments
  azure.azcollection.azure_rm_ml_batch_deployment_info:
    endpoint_name: endpoint-command-xxxxxxxxxx
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
'''

RETURN = '''
ml_batch_deployments:
    description:
        - Batch Deployments that match the query.
    returned: always
    type: dict
    sample: [
      {
          "creation_context": {
              "created_at": "2026-04-24T15:39:01.058441+00:00",
              "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "last_modified_at": "2026-04-24T15:39:01.058431+00:00"
          },
          "endpoint_name": "endpoint-command-ansible",
          "environment_variables": {},
          "error_threshold": 0,
          "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/batchEndpoints/endpoint-command-ansible/deployments/deployment-command-xxxxxxxxxx",
          "jobs": [],
          "logging_level": "Info",
          "max_concurrency_per_instance": 0,
          "mini_batch_size": 0,
          "name": "deployment-command-xxxxxxxxxx",
          "output_action": "summary_only",
          "properties": {
              "componentDeployment.Settings.continue_on_step_failure": null,
              "default_compute": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/computes/batch-cluster",
              "default_datastore": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/datastores/workspaceblobstore",
              "deployment_configuration_type": "PipelineComponent"
          },
          "provisioning_state": "Succeeded",
          "resources": {
              "instance_count": 1,
              "properties": {}
          },
          "retry_settings": {
              "max_retries": 0,
              "timeout": 0
          },
          "tags": {
              "PipelineDeployment.ComponentId": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/components/hello_batch/versions/1"
          }
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLBatchDeploymentInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            endpoint_name=dict(
                type='str',
                required=True,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_batch_deployments=[]
        )

        super(AzureRMMLBatchDeploymentInfo, self).__init__(self.module_arg_spec,
                                                           supports_tags=False,
                                                           supports_check_mode=True,
                                                           facts_module=True,
                                                           )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            ml_batch_deployments = [self.get(self.name,
                                             endpoint_name=self.endpoint_name)]
        else:
            results = self.client.batch_deployments.list(endpoint_name=self.endpoint_name)
            ml_batch_deployments = [
                self.get(x.name,
                         endpoint_name=self.endpoint_name)
                for x in results]

        self.results['ml_batch_deployments'] = ml_batch_deployments
        return self.results

    def get(self, name, endpoint_name=None):
        try:
            result = self.client.batch_deployments.get(name=name,
                                                       endpoint_name=endpoint_name)
            ml_batch_deployment = self.entity_to_dict(result)
            results = self.client.batch_deployments.list_jobs(
                name=name,
                endpoint_name=endpoint_name
            )
            ml_batch_deployment['jobs'] = [self.entity_to_dict(deployment)
                                           for deployment in results]
        except ResourceNotFoundError:
            ml_batch_deployment = None

        return ml_batch_deployment


def main():
    AzureRMMLBatchDeploymentInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2021, Kris Budde <kris@budd.ee
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: mssql_script

short_description: Execute SQL scripts on a MSSQL database

version_added: "4.0.0"

description:
  - Execute SQL scripts on a MSSQL database.
extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: partial
    details:
      - The script is not be executed in check mode.
  diff_mode:
    support: none

options:
  name:
    description: Database to run script against.
    aliases: [db]
    default: ''
    type: str
  login_user:
    description: The username used to authenticate with.
    type: str
  login_password:
    description: The password used to authenticate with.
    type: str
  login_host:
    description:
      - Host running the database.
      - For named instances, use the format V(server\\instance). In that case, do not use O(login_port).
    type: str
    required: true
  login_port:
    description:
      - Port of the MSSQL server. Requires O(login_host) to be defined as well.
      - Cannot be used together with a named instance in O(login_host) (that is, V(server\\instance) format).
      - If O(login_host) is not a named instance and O(login_port) is not specified, it defaults to V(1433).
    type: int
  script:
    description:
      - The SQL script to be executed.
      - Script can contain multiple SQL statements. Multiple Batches can be separated by V(GO) command.
      - Each batch must return at least one result set.
    required: true
    type: str
  transaction:
    description:
      - If transactional mode is requested, start a transaction and commit the change only if the script succeed. Otherwise,
        rollback the transaction.
      - If transactional mode is not requested (default), automatically commit the change.
    type: bool
    default: false
    version_added: 8.4.0
  output:
    description:
      - With V(default) each row is returned as a list of values. See RV(query_results).
      - Output format V(dict) returns dictionary with the column names as keys. See RV(query_results_dict).
      - V(dict) requires named columns to be returned by each query otherwise an error is thrown.
    choices: ["dict", "default"]
    default: 'default'
    type: str
  params:
    description: |-
      Parameters passed to the script as SQL parameters.
      (Query V('SELECT %(name\)s"') with V(example: '{"name": "John Doe"}).)'.
    type: dict
notes:
  - Requires the pymssql Python package on the remote host. For Ubuntu, this is as easy as C(pip install pymssql) (See M(ansible.builtin.pip)).
requirements:
  - pymssql

author:
  - Kris Budde (@kbudde)
"""

EXAMPLES = r"""
- name: Check DB connection
  community.general.mssql_script:
    login_user: "{{ mssql_login_user }}"
    login_password: "{{ mssql_login_password }}"
    login_host: "{{ mssql_host }}"
    login_port: "{{ mssql_port }}"
    db: master
    script: "SELECT 1"

- name: Query with parameter
  community.general.mssql_script:
    login_user: "{{ mssql_login_user }}"
    login_password: "{{ mssql_login_password }}"
    login_host: "{{ mssql_host }}"
    login_port: "{{ mssql_port }}"
    script: |
      SELECT name, state_desc FROM sys.databases WHERE name = %(dbname)s
    params:
      dbname: msdb
  register: result_params
- assert:
    that:
      - result_params.query_results[0][0][0][0] == 'msdb'
      - result_params.query_results[0][0][0][1] == 'ONLINE'

- name: Query within a transaction
  community.general.mssql_script:
    login_user: "{{ mssql_login_user }}"
    login_password: "{{ mssql_login_password }}"
    login_host: "{{ mssql_host }}"
    login_port: "{{ mssql_port }}"
    script: |
      UPDATE sys.SomeTable SET desc = 'some_table_desc' WHERE name = %(dbname)s
      UPDATE sys.AnotherTable SET desc = 'another_table_desc' WHERE name = %(dbname)s
    transaction: true
    params:
      dbname: msdb

- name: two batches with default output
  community.general.mssql_script:
    login_user: "{{ mssql_login_user }}"
    login_password: "{{ mssql_login_password }}"
    login_host: "{{ mssql_host }}"
    login_port: "{{ mssql_port }}"
    script: |
      SELECT 'Batch 0 - Select 0'
      SELECT 'Batch 0 - Select 1'
      GO
      SELECT 'Batch 1 - Select 0'
  register: result_batches
- assert:
    that:
      - result_batches.query_results | length == 2 # two batch results
      - result_batches.query_results[0] | length == 2 # two selects in first batch
      - result_batches.query_results[0][0] | length == 1 # one row in first select
      - result_batches.query_results[0][0][0] | length == 1 # one column in first row
      - result_batches.query_results[0][0][0][0] == 'Batch 0 - Select 0' # each row contains a list of values.

- name: two batches with dict output
  community.general.mssql_script:
    login_user: "{{ mssql_login_user }}"
    login_password: "{{ mssql_login_password }}"
    login_host: "{{ mssql_host }}"
    login_port: "{{ mssql_port }}"
    output: dict
    script: |
      SELECT 'Batch 0 - Select 0' as b0s0
      SELECT 'Batch 0 - Select 1' as b0s1
      GO
      SELECT 'Batch 1 - Select 0' as b1s0
  register: result_batches_dict
- assert:
    that:
      - result_batches_dict.query_results_dict | length == 2 # two batch results
      - result_batches_dict.query_results_dict[0] | length == 2 # two selects in first batch
      - result_batches_dict.query_results_dict[0][0] | length == 1 # one row in first select
      - result_batches_dict.query_results_dict[0][0][0]['b0s0'] == 'Batch 0 - Select 0' # column 'b0s0' of first row
"""

RETURN = r"""
query_results:
  description: List of batches (queries separated by V(GO) keyword).
  type: list
  elements: list
  returned: success and O(output=default)
  sample:
    [
      [
        [
          [
            "Batch 0 - Select 0"
          ]
        ],
        [
          [
            "Batch 0 - Select 1"
          ]
        ]
      ],
      [
        [
          [
            "Batch 1 - Select 0"
          ]
        ]
      ]
    ]
  contains:
    queries:
      description:
        - List of result sets of each query.
        - If a query returns no results, the results of this and all the following queries are not included in the output.
        - Use the V(GO) keyword in O(script) to separate queries.
      type: list
      elements: list
      contains:
        rows:
          description: List of rows returned by query.
          type: list
          elements: list
          contains:
            column_value:
              description:
                - List of column values.
                - Any non-standard JSON type is converted to string.
              type: list
              example: ["Batch 0 - Select 0"]
              returned: success, if output is default
query_results_dict:
  description: List of batches (queries separated by V(GO) keyword).
  type: list
  elements: list
  returned: success and O(output=dict)
  sample:
    [
      [
        [
          [
            "Batch 0 - Select 0"
          ]
        ],
        [
          [
            "Batch 0 - Select 1"
          ]
        ]
      ],
      [
        [
          [
            "Batch 1 - Select 0"
          ]
        ]
      ]
    ]
  contains:
    queries:
      description:
        - List of result sets of each query.
        - If a query returns no results, the results of this and all the following queries are not included in the output.
          Use V(GO) keyword to separate queries.
      type: list
      elements: list
      contains:
        rows:
          description: List of rows returned by query.
          type: list
          elements: list
          contains:
            column_dict:
              description:
                - Dictionary of column names and values.
                - Any non-standard JSON type is converted to string.
              type: dict
              example: {"col_name": "Batch 0 - Select 0"}
              returned: success, if output is dict
"""

import json
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib

PYMSSQL_IMP_ERR = None
try:
    import pymssql
except ImportError:
    PYMSSQL_IMP_ERR = traceback.format_exc()
    MSSQL_FOUND = False
else:
    MSSQL_FOUND = True


def clean_output(o):
    return str(o)


def run_module():
    module_args = dict(
        name=dict(aliases=["db"], default=""),
        login_user=dict(),
        login_password=dict(no_log=True),
        login_host=dict(required=True),
        login_port=dict(type="int"),
        script=dict(required=True),
        output=dict(default="default", choices=["dict", "default"]),
        params=dict(type="dict"),
        transaction=dict(type="bool", default=False),
    )

    result = dict(
        changed=False,
    )

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)
    if not MSSQL_FOUND:
        module.fail_json(msg=missing_required_lib("pymssql"), exception=PYMSSQL_IMP_ERR)

    db = module.params["name"]
    login_user = module.params["login_user"]
    login_password = module.params["login_password"]
    login_host = module.params["login_host"]
    login_port = module.params["login_port"]
    script = module.params["script"]
    output = module.params["output"]
    sql_params = module.params["params"]
    # Added param to set the transactional mode (true/false)
    transaction = module.params["transaction"]

    if "\\" in login_host and login_port is not None:
        module.fail_json(
            msg=r"login_port cannot be used with a named instance in login_host (server\instance format). "
            "Named instances use the SQL Server Browser service to resolve the port automatically."
        )

    login_querystring = login_host
    if "\\" not in login_host and login_port is not None:
        login_querystring = f"{login_host}:{login_port}"

    if login_user is not None and login_password is None:
        module.fail_json(msg="when supplying login_user argument, login_password must also be provided")

    try:
        conn = pymssql.connect(user=login_user, password=login_password, host=login_querystring, database=db)
        cursor = conn.cursor()
    except Exception as e:
        if "Unknown database" in str(e):
            errno, errstr = e.args
            module.fail_json(msg=f"ERROR: {errno} {errstr}")
        else:
            module.fail_json(
                msg="unable to connect, check login_user and login_password are correct, or alternatively check your "
                "@sysconfdir@/freetds.conf / ${HOME}/.freetds.conf"
            )

    # If transactional mode is requested, start a transaction
    conn.autocommit(not transaction)

    query_results_key = "query_results"
    if output == "dict":
        cursor = conn.cursor(as_dict=True)
        query_results_key = "query_results_dict"

    # Process the script into batches
    queries = []
    current_batch = []
    for statement in script.splitlines(True):
        # Ignore the Byte Order Mark, if found
        if statement.strip() == "\ufeff":
            continue

        # Assume each 'GO' is on its own line but may have leading/trailing whitespace
        # and be of mixed-case
        if statement.strip().upper() != "GO":
            current_batch.append(statement)
        else:
            queries.append("".join(current_batch))
            current_batch = []
    if len(current_batch) > 0:
        queries.append("".join(current_batch))

    result["changed"] = True
    if module.check_mode:
        module.exit_json(**result)

    query_results = []
    for query in queries:
        # Catch and exit on any bad query errors
        try:
            if sql_params:
                cursor.execute(query, sql_params)
            else:
                cursor.execute(query)
            qry_result = []
            rows = cursor.fetchall()
            while rows:
                qry_result.append(rows)
                rows = cursor.fetchall()
            query_results.append(qry_result)
        except Exception as e:
            # We know we executed the statement so this error just means we have no resultset
            # which is ok (eg UPDATE/INSERT)
            if (
                type(e).__name__ == "OperationalError"
                and str(e) == "Statement not executed or executed statement has no resultset"
            ):
                query_results.append([])
            else:
                # Rollback transaction before failing the module in case of error
                if transaction:
                    conn.rollback()
                error_msg = f"{type(e).__name__}: {e}"
                module.fail_json(msg="query failed", query=query, error=error_msg, **result)

    # Commit transaction before exiting the module in case of no error
    if transaction:
        conn.commit()

    # ensure that the result is json serializable
    qry_results = json.loads(json.dumps(query_results, default=clean_output))

    result[query_results_key] = qry_results
    module.exit_json(**result)


def main():
    run_module()


if __name__ == "__main__":
    main()
